@@ -218,8 +218,6 @@ def dump_sections(output):
 def dump_Makefile():
     for file in glob.glob('*.xml'):
         extras.append(file)
-    for file in glob.glob('*.res'):
-        extras.append(file)
     Makefile="""# -*- buffer-read-only: t -*- vi: set ro:
 # Beware this is autogenerated by index.py
 SUBDIRS=python
@@ -243,7 +241,7 @@ index.html: examples.xml examples.xsl $(top_srcdir)/docs/site.xsl
 install-data-local:
 	$(mkinstalldirs) $(DESTDIR)$(HTML_DIR)
 	-@INSTALL@ -m 0644 $(srcdir)/*.html $(srcdir)/*.c $(srcdir)/*.xml \\
-	  $(srcdir)/*.xsl $(srcdir)/*.res $(DESTDIR)$(HTML_DIR)
+	  $(srcdir)/*.xsl $(DESTDIR)$(HTML_DIR)
 
 """
     EXTRA_DIST= string.join(extras, ' ')
