@@ -32,7 +32,7 @@ if len(sys.argv) != 2:
 
 (name, xmldesc) = read_domain(sys.argv[1])
 
-conn = libvirt.openReadOnly(None)
+conn = libvirt.open(None)
 if conn == None:
     print 'Failed to open connection to the hypervisor'
     sys.exit(1)
