@@ -246,6 +246,11 @@ py_types = {
     'virNetwork *':  ('O', "virNetwork", "virNetworkPtr", "virNetworkPtr"),
     'const virNetwork *':  ('O', "virNetwork", "virNetworkPtr", "virNetworkPtr"),
 
+    'virInterfacePtr':  ('O', "virInterface", "virInterfacePtr", "virInterfacePtr"),
+    'const virInterfacePtr':  ('O', "virInterface", "virInterfacePtr", "virInterfacePtr"),
+    'virInterface *':  ('O', "virInterface", "virInterfacePtr", "virInterfacePtr"),
+    'const virInterface *':  ('O', "virInterface", "virInterfacePtr", "virInterfacePtr"),
+
     'virStoragePoolPtr':  ('O', "virStoragePool", "virStoragePoolPtr", "virStoragePoolPtr"),
     'const virStoragePoolPtr':  ('O', "virStoragePool", "virStoragePoolPtr", "virStoragePoolPtr"),
     'virStoragePool *':  ('O', "virStoragePool", "virStoragePoolPtr", "virStoragePoolPtr"),
@@ -289,6 +294,8 @@ skip_impl = (
     'virConnectListDefinedDomains',
     'virConnectListNetworks',
     'virConnectListDefinedNetworks',
+    'virConnectListInterfaces',
+    'virConnectListDefinedInterfaces',
     'virConnectListStoragePools',
     'virConnectListDefinedStoragePools',
     'virConnectListStorageVols',
@@ -608,6 +615,8 @@ classes_type = {
     "virDomain *": ("._o", "virDomain(self, _obj=%s)", "virDomain"),
     "virNetworkPtr": ("._o", "virNetwork(self, _obj=%s)", "virNetwork"),
     "virNetwork *": ("._o", "virNetwork(self, _obj=%s)", "virNetwork"),
+    "virInterfacePtr": ("._o", "virInterface(self, _obj=%s)", "virInterface"),
+    "virInterface *": ("._o", "virInterface(self, _obj=%s)", "virInterface"),
     "virStoragePoolPtr": ("._o", "virStoragePool(self, _obj=%s)", "virStoragePool"),
     "virStoragePool *": ("._o", "virStoragePool(self, _obj=%s)", "virStoragePool"),
     "virStorageVolPtr": ("._o", "virStorageVol(self, _obj=%s)", "virStorageVol"),
@@ -621,7 +630,8 @@ classes_type = {
 converter_type = {
 }
 
-primary_classes = ["virDomain", "virNetwork", "virStoragePool", "virStorageVol",
+primary_classes = ["virDomain", "virNetwork", "virInterface",
+                   "virStoragePool", "virStorageVol",
                    "virConnect", "virNodeDevice" ]
 
 classes_ancestor = {
@@ -629,6 +639,7 @@ classes_ancestor = {
 classes_destructors = {
     "virDomain": "virDomainFree",
     "virNetwork": "virNetworkFree",
+    "virInterface": "virInterfaceFree",
     "virStoragePool": "virStoragePoolFree",
     "virStorageVol": "virStorageVolFree",
     "virNodeDevice" : "virNodeDeviceFree"
@@ -638,6 +649,7 @@ functions_noexcept = {
     'virDomainGetID': True,
     'virDomainGetName': True,
     'virNetworkGetName': True,
+    'virInterfaceGetName': True,
     'virStoragePoolGetName': True,
     'virStorageVolGetName': True,
     'virStorageVolGetkey': True,
@@ -671,7 +683,10 @@ functions_list_exception_test = {
 functions_list_default_test = "%s is None"
 
 def is_list_type (name):
-    return name[-1:] == "*"
+    whitelist = [ "virDomainBlockStats",
+                  "virDomainInterfaceStats" ]
+
+    return name[-1:] == "*" or name in whitelist
 
 def nameFixup(name, classe, type, file):
     # avoid a desastrous clash
@@ -684,12 +699,27 @@ def nameFixup(name, classe, type, file):
     elif name[0:16] == "virNetworkDefine":
         func = name[3:]
         func = string.lower(func[0:1]) + func[1:]
+    elif name[0:19] == "virNetworkCreateXML":
+        func = name[3:]
+        func = string.lower(func[0:1]) + func[1:]
     elif name[0:16] == "virNetworkLookup":
         func = name[3:]
         func = string.lower(func[0:1]) + func[1:]
+    elif name[0:18] == "virInterfaceDefine":
+        func = name[3:]
+        func = string.lower(func[0:1]) + func[1:]
+    elif name[0:21] == "virInterfaceCreateXML":
+        func = name[3:]
+        func = string.lower(func[0:1]) + func[1:]
+    elif name[0:18] == "virInterfaceLookup":
+        func = name[3:]
+        func = string.lower(func[0:1]) + func[1:]
     elif name[0:20] == "virStoragePoolDefine":
         func = name[3:]
         func = string.lower(func[0:1]) + func[1:]
+    elif name[0:23] == "virStoragePoolCreateXML":
+        func = name[3:]
+        func = string.lower(func[0:1]) + func[1:]
     elif name[0:20] == "virStoragePoolLookup":
         func = name[3:]
         func = string.lower(func[0:1]) + func[1:]
@@ -711,6 +741,12 @@ def nameFixup(name, classe, type, file):
     elif name[0:10] == "virNetwork":
         func = name[10:]
         func = string.lower(func[0:1]) + func[1:]
+    elif name[0:15] == "virInterfaceGet":
+        func = name[13:]
+        func = string.lower(func[0:1]) + func[1:]
+    elif name[0:12] == "virInterface":
+        func = name[10:]
+        func = string.lower(func[0:1]) + func[1:]
     elif name[0:17] == "virStoragePoolGet":
         func = name[17:]
         func = string.lower(func[0:1]) + func[1:]
@@ -982,7 +1018,7 @@ def buildWrappers():
 	    else:
 		txt.write("Class %s()\n" % (classname))
 		classes.write("class %s:\n" % (classname))
-                if classname in [ "virDomain", "virNetwork", "virStoragePool", "virStorageVol", "virNodeDevice" ]:
+                if classname in [ "virDomain", "virNetwork", "virInterface", "virStoragePool", "virStorageVol", "virNodeDevice" ]:
                     classes.write("    def __init__(self, conn, _obj=None):\n")
                 else:
                     classes.write("    def __init__(self, _obj=None):\n")
@@ -990,7 +1026,7 @@ def buildWrappers():
 		    list = reference_keepers[classname]
 		    for ref in list:
 		        classes.write("        self.%s = None\n" % ref[1])
-                if classname in [ "virDomain", "virNetwork", "virNodeDevice" ]:
+                if classname in [ "virDomain", "virNetwork", "virInterface", "virNodeDevice" ]:
                     classes.write("        self._conn = conn\n")
                 elif classname in [ "virStorageVol", "virStoragePool" ]:
                     classes.write("        self._conn = conn\n" + \
@@ -1066,6 +1102,9 @@ def buildWrappers():
 		    n = n + 1
 		classes.write(")\n");
 
+                if name == "virConnectClose":
+                    classes.write("        self._o = None\n")
+
                 # For functions returning object types:
                 if ret[0] != "void":
 		    if classes_type.has_key(ret[0]):
@@ -1086,6 +1125,10 @@ def buildWrappers():
                                               (name))
                             elif classname == "virNetwork":
                                 classes.write(
+		     "        if ret is None:raise libvirtError('%s() failed', net=self)\n" %
+                                              (name))
+                            elif classname == "virInterface":
+                                classes.write(
 		     "        if ret is None:raise libvirtError('%s() failed', net=self)\n" %
                                               (name))
                             elif classname == "virStoragePool":
@@ -1168,6 +1211,10 @@ def buildWrappers():
                                 classes.write (("        if " + test +
                                                 ": raise libvirtError ('%s() failed', net=self)\n") %
                                                ("ret", name))
+                            elif classname == "virInterface":
+                                classes.write (("        if " + test +
+                                                ": raise libvirtError ('%s() failed', net=self)\n") %
+                                               ("ret", name))
                             elif classname == "virStoragePool":
                                 classes.write (("        if " + test +
                                                 ": raise libvirtError ('%s() failed', pool=self)\n") %
@@ -1206,6 +1253,10 @@ def buildWrappers():
                                 classes.write (("        if " + test +
                                                 ": raise libvirtError ('%s() failed', net=self)\n") %
                                                ("ret", name))
+                            elif classname == "virInterface":
+                                classes.write (("        if " + test +
+                                                ": raise libvirtError ('%s() failed', net=self)\n") %
+                                               ("ret", name))
                             elif classname == "virStoragePool":
                                 classes.write (("        if " + test +
                                                 ": raise libvirtError ('%s() failed', pool=self)\n") %
