@@ -48,6 +48,15 @@ typedef struct {
 } PyvirNetwork_Object;
 
 
+#define PyvirInterface_Get(v) (((v) == Py_None) ? NULL : \
+        (((PyvirInterface_Object *)(v))->obj))
+
+typedef struct {
+    PyObject_HEAD
+    virInterfacePtr obj;
+} PyvirInterface_Object;
+
+
 #define PyvirStoragePool_Get(v) (((v) == Py_None) ? NULL : \
         (((PyvirStoragePool_Object *)(v))->obj))
 
@@ -118,6 +127,7 @@ PyObject * libvirt_charPtrConstWrap(const char *str);
 PyObject * libvirt_virConnectPtrWrap(virConnectPtr node);
 PyObject * libvirt_virDomainPtrWrap(virDomainPtr node);
 PyObject * libvirt_virNetworkPtrWrap(virNetworkPtr node);
+PyObject * libvirt_virInterfacePtrWrap(virInterfacePtr node);
 PyObject * libvirt_virStoragePoolPtrWrap(virStoragePoolPtr node);
 PyObject * libvirt_virStorageVolPtrWrap(virStorageVolPtr node);
 PyObject * libvirt_virEventHandleCallbackWrap(virEventHandleCallback node);
@@ -175,4 +185,3 @@ PyObject * libvirt_virNodeDevicePtrWrap(virNodeDevicePtr node);
   if (PyEval_ThreadsInitialized())			       \
     PyGILState_Release(_save);				       \
   } LIBVIRT_STMT_END
-
