@@ -118,6 +118,21 @@ libvirt_virNetworkPtrWrap(virNetworkPtr node)
     return (ret);
 }
 
+PyObject *
+libvirt_virInterfacePtrWrap(virInterfacePtr node)
+{
+    PyObject *ret;
+
+    if (node == NULL) {
+        Py_INCREF(Py_None);
+        return (Py_None);
+    }
+    ret =
+        PyCObject_FromVoidPtrAndDesc((void *) node, (char *) "virInterfacePtr",
+                                     NULL);
+    return (ret);
+}
+
 PyObject *
 libvirt_virStoragePoolPtrWrap(virStoragePoolPtr node)
 {
