@@ -49,8 +49,6 @@
 #include "util.h"
 #include "logging.h"
 
-#define MAX_TAP_ID 256
-
 #define JIFFIES_TO_MS(j) (((j)*1000)/HZ)
 #define MS_TO_JIFFIES(ms) (((ms)*HZ)/1000)
 
@@ -139,7 +137,7 @@ brAddBridge(brControl *ctl,
 }
 #else
 int brAddBridge (brControl *ctl ATTRIBUTE_UNUSED,
-                 char **name ATTRIBUTE_UNUSED)
+                 const char *name ATTRIBUTE_UNUSED)
 {
     return EINVAL;
 }
@@ -175,8 +173,8 @@ brHasBridge(brControl *ctl,
 }
 #else
 int
-brHasBridge(brControl *ctl,
-            const char *name)
+brHasBridge(brControl *ctl ATTRIBUTE_UNUSED,
+            const char *name ATTRIBUTE_UNUSED)
 {
     return EINVAL;
 }
@@ -403,6 +401,7 @@ static int brSetInterfaceMtu(brControl *ctl,
  *
  * Returns 0 in case of success or an errno code in case of failure.
  */
+#ifdef IFF_VNET_HDR
 static int
 brProbeVnetHdr(int tapfd)
 {
@@ -440,6 +439,7 @@ brProbeVnetHdr(int tapfd)
     return 0;
 #endif
 }
+#endif
 
 /**
  * brAddTap:
@@ -451,8 +451,11 @@ brProbeVnetHdr(int tapfd)
  *
  * This function creates a new tap device on a bridge. @ifname can be either
  * a fixed name or a name template with '%d' for dynamic name allocation.
- * in either case the final name for the bridge will be stored in @ifname
- * and the associated file descriptor in @tapfd.
+ * in either case the final name for the bridge will be stored in @ifname.
+ * If the @tapfd parameter is supplied, the open tap device file
+ * descriptor will be returned, otherwise the TAP device will be made
+ * persistent and closed. The caller must use brDeleteTap to remove
+ * a persistent TAP devices when it is no longer needed.
  *
  * Returns 0 in case of success or an errno code in case of failure.
  */
@@ -463,72 +466,56 @@ brAddTap(brControl *ctl,
          int vnet_hdr,
          int *tapfd)
 {
-    int id, subst, fd;
+    int fd, len;
+    struct ifreq ifr;
 
-    if (!ctl || !ctl->fd || !bridge || !ifname || !tapfd)
+    if (!ctl || !ctl->fd || !bridge || !ifname)
         return EINVAL;
 
-    subst = id = 0;
-
-    if (strstr(*ifname, "%d"))
-        subst = 1;
-
     if ((fd = open("/dev/net/tun", O_RDWR)) < 0)
       return errno;
 
-    if (vnet_hdr)
-        vnet_hdr = brProbeVnetHdr(fd);
+    memset(&ifr, 0, sizeof(ifr));
 
-    do {
-        struct ifreq try;
-        int len;
-
-        memset(&try, 0, sizeof(struct ifreq));
-
-        try.ifr_flags = IFF_TAP|IFF_NO_PI;
+    ifr.ifr_flags = IFF_TAP|IFF_NO_PI;
 
 #ifdef IFF_VNET_HDR
-        if (vnet_hdr)
-            try.ifr_flags |= IFF_VNET_HDR;
+    if (vnet_hdr && brProbeVnetHdr(fd))
+        ifr.ifr_flags |= IFF_VNET_HDR;
+#else
+    (void) vnet_hdr;
 #endif
 
-        if (subst) {
-            len = snprintf(try.ifr_name, BR_IFNAME_MAXLEN, *ifname, id);
-            if (len >= BR_IFNAME_MAXLEN) {
-                errno = EADDRINUSE;
-                goto error;
-            }
-        } else {
-            len = strlen(*ifname);
-            if (len >= BR_IFNAME_MAXLEN - 1) {
-                errno = EINVAL;
-                goto error;
-            }
+    strncpy(ifr.ifr_name, *ifname, IFNAMSIZ-1);
 
-            strncpy(try.ifr_name, *ifname, len);
-            try.ifr_name[len] = '\0';
-        }
+    if (ioctl(fd, TUNSETIFF, &ifr) < 0)
+        goto error;
 
-        if (ioctl(fd, TUNSETIFF, &try) == 0) {
-            /* We need to set the interface MTU before adding it
-             * to the bridge, because the bridge will have its
-             * MTU adjusted automatically when we add the new interface.
-             */
-            if ((errno = brSetInterfaceMtu(ctl, bridge, try.ifr_name)))
-                goto error;
-            if ((errno = brAddInterface(ctl, bridge, try.ifr_name)))
-                goto error;
-            if ((errno = brSetInterfaceUp(ctl, try.ifr_name, 1)))
-                goto error;
-            VIR_FREE(*ifname);
-            if (!(*ifname = strdup(try.ifr_name)))
-                goto error;
-            *tapfd = fd;
-            return 0;
-        }
+    len = strlen(ifr.ifr_name);
+    if (len >= BR_IFNAME_MAXLEN - 1) {
+        errno = EINVAL;
+        goto error;
+    }
 
-        id++;
-    } while (subst && id <= MAX_TAP_ID);
+    /* We need to set the interface MTU before adding it
+     * to the bridge, because the bridge will have its
+     * MTU adjusted automatically when we add the new interface.
+     */
+    if ((errno = brSetInterfaceMtu(ctl, bridge, ifr.ifr_name)))
+        goto error;
+    if ((errno = brAddInterface(ctl, bridge, ifr.ifr_name)))
+        goto error;
+    if ((errno = brSetInterfaceUp(ctl, ifr.ifr_name, 1)))
+        goto error;
+    if (!tapfd &&
+        (errno = ioctl(fd, TUNSETPERSIST, 1)))
+        goto error;
+    VIR_FREE(*ifname);
+    if (!(*ifname = strdup(ifr.ifr_name)))
+        goto error;
+    if (tapfd)
+        *tapfd = fd;
+    return 0;
 
  error:
     close(fd);
@@ -536,6 +523,43 @@ brAddTap(brControl *ctl,
     return errno;
 }
 
+int brDeleteTap(brControl *ctl,
+                const char *ifname)
+{
+    struct ifreq try;
+    int len;
+    int fd;
+
+    if (!ctl || !ctl->fd || !ifname)
+        return EINVAL;
+
+    if ((fd = open("/dev/net/tun", O_RDWR)) < 0)
+        return errno;
+
+    memset(&try, 0, sizeof(struct ifreq));
+    try.ifr_flags = IFF_TAP|IFF_NO_PI;
+
+    len = strlen(ifname);
+    if (len >= BR_IFNAME_MAXLEN - 1) {
+        errno = EINVAL;
+        goto error;
+    }
+
+    strncpy(try.ifr_name, ifname, len);
+    try.ifr_name[len] = '\0';
+
+    if (ioctl(fd, TUNSETIFF, &try) == 0) {
+        if ((errno = ioctl(fd, TUNSETPERSIST, 0)))
+            goto error;
+    }
+
+ error:
+    close(fd);
+
+    return errno;
+}
+
+
 /**
  * brSetInterfaceUp:
  * @ctl: bridge control pointer
