@@ -35,13 +35,21 @@ struct _virCapsGuestFeature {
     int toggle;
 };
 
+typedef struct _virCapsGuestMachine virCapsGuestMachine;
+typedef virCapsGuestMachine *virCapsGuestMachinePtr;
+struct _virCapsGuestMachine {
+    char *name;
+    char *canonical;
+};
+
 typedef struct _virCapsGuestDomainInfo virCapsGuestDomainInfo;
 typedef virCapsGuestDomainInfo *virCapsGuestDomainInfoPtr;
 struct _virCapsGuestDomainInfo {
     char *emulator;
     char *loader;
     int nmachines;
-    char **machines;
+    virCapsGuestMachinePtr *machines;
+    time_t emulator_mtime; /* do @machines need refreshing? */
 };
 
 typedef struct _virCapsGuestDomain virCapsGuestDomain;
@@ -106,6 +114,7 @@ struct _virCaps {
     int nguests;
     virCapsGuestPtr *guests;
     unsigned char macPrefix[VIR_MAC_PREFIX_BUFLEN];
+    unsigned int emulatorRequired : 1;
 };
 
 
@@ -117,6 +126,9 @@ virCapabilitiesNew(const char *arch,
 extern void
 virCapabilitiesFree(virCapsPtr caps);
 
+extern void
+virCapabilitiesFreeNUMAInfo(virCapsPtr caps);
+
 extern void
 virCapabilitiesSetMacPrefix(virCapsPtr caps,
                             unsigned char *prefix);
@@ -125,6 +137,12 @@ extern void
 virCapabilitiesGenerateMac(virCapsPtr caps,
                            unsigned char *mac);
 
+extern void
+virCapabilitiesSetEmulatorRequired(virCapsPtr caps);
+
+extern unsigned int
+virCapabilitiesIsEmulatorRequired(virCapsPtr caps);
+
 extern int
 virCapabilitiesAddHostFeature(virCapsPtr caps,
                               const char *name);
@@ -142,6 +160,13 @@ virCapabilitiesAddHostNUMACell(virCapsPtr caps,
 
 
 
+extern virCapsGuestMachinePtr *
+virCapabilitiesAllocMachines(const char *const *names,
+                             int nnames);
+extern void
+virCapabilitiesFreeMachines(virCapsGuestMachinePtr *machines,
+                            int nmachines);
+
 extern virCapsGuestPtr
 virCapabilitiesAddGuest(virCapsPtr caps,
                         const char *ostype,
@@ -150,7 +175,7 @@ virCapabilitiesAddGuest(virCapsPtr caps,
                         const char *emulator,
                         const char *loader,
                         int nmachines,
-                        const char *const *machines);
+                        virCapsGuestMachinePtr *machines);
 
 extern virCapsGuestDomainPtr
 virCapabilitiesAddGuestDomain(virCapsGuestPtr guest,
@@ -158,7 +183,7 @@ virCapabilitiesAddGuestDomain(virCapsGuestPtr guest,
                               const char *emulator,
                               const char *loader,
                               int nmachines,
-                              const char *const *machines);
+                              virCapsGuestMachinePtr *machines);
 
 extern virCapsGuestFeaturePtr
 virCapabilitiesAddGuestFeature(virCapsGuestPtr guest,
@@ -177,7 +202,8 @@ virCapabilitiesSupportsGuestArch(virCapsPtr caps,
 
 extern const char *
 virCapabilitiesDefaultGuestArch(virCapsPtr caps,
-                                const char *ostype);
+                                const char *ostype,
+                                const char *domain);
 extern const char *
 virCapabilitiesDefaultGuestMachine(virCapsPtr caps,
                                    const char *ostype,
