@@ -33,6 +33,11 @@
 #include "util.h"
 #include "threads.h"
 
+/* Private component of virDomainXMLFlags */
+typedef enum {
+   VIR_DOMAIN_XML_INTERNAL_STATUS = (1<<16), /* dump internal domain status information */
+} virDomainXMLInternalFlags;
+
 /* Different types of hypervisor */
 /* NB: Keep in sync with virDomainVirtTypeToString impl */
 enum virDomainVirtType {
@@ -48,6 +53,9 @@ enum virDomainVirtType {
     VIR_DOMAIN_VIRT_TEST,
     VIR_DOMAIN_VIRT_VMWARE,
     VIR_DOMAIN_VIRT_HYPERV,
+    VIR_DOMAIN_VIRT_VBOX,
+    VIR_DOMAIN_VIRT_ONE,
+    VIR_DOMAIN_VIRT_PHYP,
 
     VIR_DOMAIN_VIRT_LAST,
 };
@@ -104,9 +112,19 @@ struct _virDomainDiskDef {
     int cachemode;
     unsigned int readonly : 1;
     unsigned int shared : 1;
-    int slotnum; /* pci slot number for unattach */
+    struct {
+        unsigned domain;
+        unsigned bus;
+        unsigned slot;
+    } pci_addr;
 };
 
+static inline int
+virDiskHasValidPciAddr(virDomainDiskDefPtr def)
+{
+    return def->pci_addr.domain || def->pci_addr.domain || def->pci_addr.slot;
+}
+
 
 /* Two types of disk backends */
 enum virDomainFSType {
@@ -137,6 +155,7 @@ enum virDomainNetType {
     VIR_DOMAIN_NET_TYPE_MCAST,
     VIR_DOMAIN_NET_TYPE_NETWORK,
     VIR_DOMAIN_NET_TYPE_BRIDGE,
+    VIR_DOMAIN_NET_TYPE_INTERNAL,
 
     VIR_DOMAIN_NET_TYPE_LAST,
 };
@@ -167,10 +186,27 @@ struct _virDomainNetDef {
             char *script;
             char *ipaddr;
         } bridge;
+        struct {
+            char *name;
+        } internal;
     } data;
     char *ifname;
+    char *nic_name;
+    char *hostnet_name;
+    struct {
+        unsigned domain;
+        unsigned bus;
+        unsigned slot;
+    } pci_addr;
+    int vlan;
 };
 
+static inline int
+virNetHasValidPciAddr(virDomainNetDefPtr def)
+{
+    return def->pci_addr.domain || def->pci_addr.domain || def->pci_addr.slot;
+}
+
 enum virDomainChrSrcType {
     VIR_DOMAIN_CHR_TYPE_NULL,
     VIR_DOMAIN_CHR_TYPE_VC,
@@ -248,7 +284,7 @@ enum virDomainSoundModel {
     VIR_DOMAIN_SOUND_MODEL_SB16,
     VIR_DOMAIN_SOUND_MODEL_ES1370,
     VIR_DOMAIN_SOUND_MODEL_PCSPK,
-    VIR_DOMAIN_SOUND_MODEL_ES97,
+    VIR_DOMAIN_SOUND_MODEL_AC97,
 
     VIR_DOMAIN_SOUND_MODEL_LAST
 };
@@ -259,10 +295,32 @@ struct _virDomainSoundDef {
     int model;
 };
 
+
+enum virDomainVideoType {
+    VIR_DOMAIN_VIDEO_TYPE_VGA,
+    VIR_DOMAIN_VIDEO_TYPE_CIRRUS,
+    VIR_DOMAIN_VIDEO_TYPE_VMVGA,
+    VIR_DOMAIN_VIDEO_TYPE_XEN,
+    VIR_DOMAIN_VIDEO_TYPE_VBOX,
+
+    VIR_DOMAIN_VIDEO_TYPE_LAST
+};
+
+
+typedef struct _virDomainVideoDef virDomainVideoDef;
+typedef virDomainVideoDef *virDomainVideoDefPtr;
+struct _virDomainVideoDef {
+    int type;
+    unsigned int vram;
+    unsigned int heads;
+};
+
 /* 3 possible graphics console modes */
 enum virDomainGraphicsType {
     VIR_DOMAIN_GRAPHICS_TYPE_SDL,
     VIR_DOMAIN_GRAPHICS_TYPE_VNC,
+    VIR_DOMAIN_GRAPHICS_TYPE_RDP,
+    VIR_DOMAIN_GRAPHICS_TYPE_DESKTOP,
 
     VIR_DOMAIN_GRAPHICS_TYPE_LAST,
 };
@@ -284,6 +342,17 @@ struct _virDomainGraphicsDef {
             char *xauth;
             int fullscreen;
         } sdl;
+        struct {
+            int port;
+            char *listenAddr;
+            int autoport : 1;
+            int replaceUser : 1;
+            int multiUser : 1;
+        } rdp;
+        struct {
+            char *display;
+            int fullscreen : 1;
+        } desktop;
     } data;
 };
 
@@ -342,6 +411,7 @@ enum virDomainDeviceType {
     VIR_DOMAIN_DEVICE_NET,
     VIR_DOMAIN_DEVICE_INPUT,
     VIR_DOMAIN_DEVICE_SOUND,
+    VIR_DOMAIN_DEVICE_VIDEO,
     VIR_DOMAIN_DEVICE_HOSTDEV,
 
     VIR_DOMAIN_DEVICE_LAST,
@@ -357,6 +427,7 @@ struct _virDomainDeviceDef {
         virDomainNetDefPtr net;
         virDomainInputDefPtr input;
         virDomainSoundDefPtr sound;
+        virDomainVideoDefPtr video;
         virDomainHostdevDefPtr hostdev;
     } data;
 };
@@ -455,8 +526,8 @@ struct _virDomainDef {
 
     int localtime;
 
-    /* Only 1 */
-    virDomainGraphicsDefPtr graphics;
+    int ngraphics;
+    virDomainGraphicsDefPtr *graphics;
 
     int ndisks;
     virDomainDiskDefPtr *disks;
@@ -473,6 +544,9 @@ struct _virDomainDef {
     int nsounds;
     virDomainSoundDefPtr *sounds;
 
+    int nvideos;
+    virDomainVideoDefPtr *videos;
+
     int nhostdevs;
     virDomainHostdevDefPtr *hostdevs;
 
@@ -494,9 +568,8 @@ struct _virDomainObj {
     virMutex lock;
 
     int monitor;
-    char *monitorpath;
+    virDomainChrDefPtr monitor_chr;
     int monitorWatch;
-    int logfile;
     int pid;
     int state;
 
@@ -539,6 +612,7 @@ void virDomainFSDefFree(virDomainFSDefPtr def);
 void virDomainNetDefFree(virDomainNetDefPtr def);
 void virDomainChrDefFree(virDomainChrDefPtr def);
 void virDomainSoundDefFree(virDomainSoundDefPtr def);
+void virDomainVideoDefFree(virDomainVideoDefPtr def);
 void virDomainHostdevDefFree(virDomainHostdevDefPtr def);
 void virDomainDeviceDefFree(virDomainDeviceDefPtr def);
 void virDomainDefFree(virDomainDefPtr vm);
@@ -570,10 +644,22 @@ virDomainDefPtr virDomainDefParseNode(virConnectPtr conn,
                                       xmlDocPtr doc,
                                       xmlNodePtr root,
                                       int flags);
+
+virDomainObjPtr virDomainObjParseFile(virConnectPtr conn,
+                                      virCapsPtr caps,
+                                      const char *filename);
+virDomainObjPtr virDomainObjParseNode(virConnectPtr conn,
+                                      virCapsPtr caps,
+                                      xmlDocPtr xml,
+                                      xmlNodePtr root);
+
 #endif
 char *virDomainDefFormat(virConnectPtr conn,
                          virDomainDefPtr def,
                          int flags);
+char *virDomainObjFormat(virConnectPtr conn,
+                         virDomainObjPtr obj,
+                         int flags);
 
 int virDomainCpuSetParse(virConnectPtr conn,
                          const char **str,
@@ -596,6 +682,9 @@ int virDomainSaveXML(virConnectPtr conn,
 int virDomainSaveConfig(virConnectPtr conn,
                         const char *configDir,
                         virDomainDefPtr def);
+int virDomainSaveStatus(virConnectPtr conn,
+                        const char *statusDir,
+                        virDomainObjPtr obj);
 
 typedef void (*virDomainLoadConfigNotify)(virDomainObjPtr dom,
                                           int newDomain,
@@ -615,6 +704,7 @@ int virDomainLoadAllConfigs(virConnectPtr conn,
                             virDomainObjListPtr doms,
                             const char *configDir,
                             const char *autostartDir,
+                            int liveStatus,
                             virDomainLoadConfigNotify notify,
                             void *opaque);
 
@@ -631,9 +721,9 @@ int virDiskNameToBusDeviceIndex(virDomainDiskDefPtr disk,
                                 int *busIdx,
                                 int *devIdx);
 
-const char *virDomainDefDefaultEmulator(virConnectPtr conn,
-                                        virDomainDefPtr def,
-                                        virCapsPtr caps);
+virDomainFSDefPtr virDomainGetRootFilesystem(virDomainDefPtr def);
+int virDomainVideoDefaultType(virDomainDefPtr def);
+int virDomainVideoDefaultRAM(virDomainDefPtr def, int type);
 
 void virDomainObjLock(virDomainObjPtr obj);
 void virDomainObjUnlock(virDomainObjPtr obj);
@@ -651,6 +741,7 @@ VIR_ENUM_DECL(virDomainFS)
 VIR_ENUM_DECL(virDomainNet)
 VIR_ENUM_DECL(virDomainChr)
 VIR_ENUM_DECL(virDomainSoundModel)
+VIR_ENUM_DECL(virDomainVideo)
 VIR_ENUM_DECL(virDomainHostdevMode)
 VIR_ENUM_DECL(virDomainHostdevSubsys)
 VIR_ENUM_DECL(virDomainInput)
