@@ -39,9 +39,6 @@
 #include "util.h"
 #include "threads.h"
 
-#ifdef ENABLE_DEBUG
-int debugFlag = 0;
-
 /*
  * Macro used to format the message as a string in virLogMessage
  * and borrowed from libxml2 (also used in virRaiseError)
@@ -122,7 +119,7 @@ static int virLogNbOutputs = 0;
 /*
  * Default priorities
  */
-static virLogPriority virLogDefaultPriority = VIR_LOG_WARN;
+static virLogPriority virLogDefaultPriority = VIR_LOG_DEFAULT;
 
 static int virLogResetFilters(void);
 static int virLogResetOutputs(void);
@@ -177,7 +174,7 @@ int virLogStartup(void) {
     virLogLen = 0;
     virLogStart = 0;
     virLogEnd = 0;
-    virLogDefaultPriority = VIR_LOG_WARN;
+    virLogDefaultPriority = VIR_LOG_DEFAULT;
     virLogUnlock();
     return(0);
 }
@@ -199,7 +196,7 @@ int virLogReset(void) {
     virLogLen = 0;
     virLogStart = 0;
     virLogEnd = 0;
-    virLogDefaultPriority = VIR_LOG_WARN;
+    virLogDefaultPriority = VIR_LOG_DEFAULT;
     virLogUnlock();
     return(0);
 }
@@ -516,7 +513,7 @@ void virLogMessage(const char *category, int priority, const char *funcname,
     gettimeofday(&cur_time, NULL);
     localtime_r(&cur_time.tv_sec, &time_info);
 
-    if ((funcname != NULL) && (priority == VIR_LOG_DEBUG)) {
+    if ((funcname != NULL)) {
         ret = virAsprintf(&msg, "%02d:%02d:%02d.%03d: %s : %s:%lld : %s\n",
                           time_info.tm_hour, time_info.tm_min,
                           time_info.tm_sec, (int) cur_time.tv_usec / 1000,
@@ -692,7 +689,7 @@ int virLogParseOutputs(const char *outputs) {
     virSkipSpaces(&cur);
     while (*cur != 0) {
         prio= virParseNumber(&cur);
-        if ((prio < 0) || (prio > 4))
+        if ((prio < VIR_LOG_DEBUG) || (prio > VIR_LOG_ERROR))
             return(-1);
         if (*cur != ':')
             return(-1);
@@ -773,7 +770,7 @@ int virLogParseFilters(const char *filters) {
     virSkipSpaces(&cur);
     while (*cur != 0) {
         prio= virParseNumber(&cur);
-        if ((prio < 0) || (prio > 4))
+        if ((prio < VIR_LOG_DEBUG) || (prio > VIR_LOG_ERROR))
             return(-1);
         if (*cur != ':')
             return(-1);
@@ -793,5 +790,3 @@ int virLogParseFilters(const char *filters) {
     }
     return(ret);
 }
-#endif /* ENABLE_DEBUG */
-
