@@ -46,7 +46,7 @@ virCapsPtr lxcCapsInit(void)
                                    0, 0)) == NULL)
         goto no_memory;
 
-    if (virCapsInitNUMA(caps) < 0)
+    if (nodeCapsInitNUMA(caps) < 0)
         goto no_memory;
 
     /* XXX shouldn't 'borrow' KVM's prefix */
@@ -69,6 +69,10 @@ virCapsPtr lxcCapsInit(void)
                                       0,
                                       NULL) == NULL)
         goto no_memory;
+
+    /* LXC Requires an emulator in the XML */
+    virCapabilitiesSetEmulatorRequired(caps);
+
     return caps;
 
 no_memory:
