@@ -592,6 +592,9 @@ char *virNetworkDefFormat(virConnectPtr conn,
                       def->stp ? "on" : "off",
                       def->delay);
 
+    if (def->domain)
+        virBufferVSprintf(&buf, "  <domain name='%s'/>\n", def->domain);
+
     if (def->ipAddress || def->netmask) {
         virBufferAddLit(&buf, "  <ip");
 
@@ -744,10 +747,10 @@ virNetworkObjPtr virNetworkLoadConfig(virConnectPtr conn,
         goto error;
     }
 
-    /* Generate a bridge if none is found, but don't check for collisions
+    /* Generate a bridge if none is specified, but don't check for collisions
      * if a bridge is hardcoded, so the network is at least defined
      */
-    if (!def->bridge && !(def->bridge = virNetworkAllocateBridge(conn, nets)))
+    if (virNetworkSetBridgeName(conn, nets, def, 0))
         goto error;
 
     if (!(net = virNetworkAssignDef(conn, nets, def)))
@@ -875,16 +878,20 @@ int virNetworkBridgeInUse(const virNetworkObjListPtr nets,
 }
 
 char *virNetworkAllocateBridge(virConnectPtr conn,
-                               const virNetworkObjListPtr nets)
+                               const virNetworkObjListPtr nets,
+                               const char *template)
 {
 
     int id = 0;
     char *newname;
 
+    if (!template)
+        template = "virbr%d";
+
     do {
         char try[50];
 
-        snprintf(try, sizeof(try), "virbr%d", id);
+        snprintf(try, sizeof(try), template, id);
 
         if (!virNetworkBridgeInUse(nets, try, NULL)) {
             if (!(newname = strdup(try))) {
@@ -905,12 +912,17 @@ char *virNetworkAllocateBridge(virConnectPtr conn,
 
 int virNetworkSetBridgeName(virConnectPtr conn,
                             const virNetworkObjListPtr nets,
-                            virNetworkDefPtr def) {
+                            virNetworkDefPtr def,
+                            int check_collision) {
 
     int ret = -1;
 
-    if (def->bridge) {
-        if (virNetworkBridgeInUse(nets, def->bridge, def->name)) {
+    if (def->bridge && !strstr(def->bridge, "%d")) {
+        /* We may want to skip collision detection in this case (ex. when
+         * loading configs at daemon startup, so the network is at least
+         * defined. */
+        if (check_collision &&
+            virNetworkBridgeInUse(nets, def->bridge, def->name)) {
             networkReportError(conn, NULL, NULL, VIR_ERR_INTERNAL_ERROR,
                                _("bridge name '%s' already in use."),
                                def->bridge);
@@ -918,7 +930,7 @@ int virNetworkSetBridgeName(virConnectPtr conn,
         }
     } else {
         /* Allocate a bridge name */
-        if (!(def->bridge = virNetworkAllocateBridge(conn, nets)))
+        if (!(def->bridge = virNetworkAllocateBridge(conn, nets, def->bridge)))
             goto error;
     }
 
@@ -936,4 +948,3 @@ void virNetworkObjUnlock(virNetworkObjPtr obj)
 {
     virMutexUnlock(&obj->lock);
 }
-
