@@ -174,14 +174,15 @@ int virNetworkBridgeInUse(const virNetworkObjListPtr nets,
                           const char *skipname);
 
 char *virNetworkAllocateBridge(virConnectPtr conn,
-                               const virNetworkObjListPtr nets);
+                               const virNetworkObjListPtr nets,
+                               const char *template);
 
 int virNetworkSetBridgeName(virConnectPtr conn,
                             const virNetworkObjListPtr nets,
-                            virNetworkDefPtr def);
+                            virNetworkDefPtr def,
+                            int check_collision);
 
 void virNetworkObjLock(virNetworkObjPtr obj);
 void virNetworkObjUnlock(virNetworkObjPtr obj);
 
 #endif /* __NETWORK_CONF_H__ */
-
