@@ -53,8 +53,33 @@ VIR_ENUM_IMPL(virNodeDevNetCap, VIR_NODE_DEV_CAP_NET_LAST,
               "80203",
               "80211")
 
+VIR_ENUM_IMPL(virNodeDevHBACap, VIR_NODE_DEV_CAP_HBA_LAST,
+              "fc_host",
+              "vport_ops")
+
+
+static int
+virNodeDevCapsDefParseString(virConnectPtr conn,
+                             const char *xpath,
+                             xmlXPathContextPtr ctxt,
+                             char **string,
+                             virNodeDeviceDefPtr def,
+                             const char *missing_error_fmt)
+{
+    char *s;
+
+    s = virXPathString(conn, xpath, ctxt);
+    if (s == NULL) {
+        virNodeDeviceReportError(conn, VIR_ERR_INTERNAL_ERROR,
+                                 missing_error_fmt,
+                                 def->name);
+        return -1;
+    }
+
+    *string = s;
+    return 0;
+}
 
-#define virNodeDeviceLog(msg...) fprintf(stderr, msg)
 
 virNodeDeviceObjPtr virNodeDeviceFindByName(const virNodeDeviceObjListPtr devs,
                                             const char *name)
@@ -81,6 +106,7 @@ void virNodeDeviceDefFree(virNodeDeviceDefPtr def)
 
     VIR_FREE(def->name);
     VIR_FREE(def->parent);
+    VIR_FREE(def->driver);
 
     caps = def->caps;
     while (caps) {
@@ -97,6 +123,7 @@ void virNodeDeviceObjFree(virNodeDeviceObjPtr dev)
     if (!dev)
         return;
 
+    VIR_FREE(dev->devicePath);
     virNodeDeviceDefFree(dev->def);
     if (dev->privateFree)
         (*dev->privateFree)(dev->privateData);
@@ -194,6 +221,11 @@ char *virNodeDeviceDefFormat(virConnectPtr conn,
 
     if (def->parent)
         virBufferEscapeString(&buf, "  <parent>%s</parent>\n", def->parent);
+    if (def->driver) {
+        virBufferAddLit(&buf, "  <driver>\n");
+        virBufferEscapeString(&buf, "    <name>%s</name>\n", def->driver);
+        virBufferAddLit(&buf, "  </driver>\n");
+    }
 
     for (caps = def->caps; caps; caps = caps->next) {
         char uuidstr[VIR_UUID_STRING_BUFLEN];
@@ -302,6 +334,18 @@ char *virNodeDeviceDefFormat(virConnectPtr conn,
         case VIR_NODE_DEV_CAP_SCSI_HOST:
             virBufferVSprintf(&buf, "    <host>%d</host>\n",
                               data->scsi_host.host);
+            if (data->scsi_host.flags & VIR_NODE_DEV_CAP_FLAG_HBA_FC_HOST) {
+                virBufferAddLit(&buf, "    <capability type='fc_host'>\n");
+                virBufferVSprintf(&buf,
+                                  "      <wwnn>%s</wwnn>\n", data->scsi_host.wwnn);
+                virBufferVSprintf(&buf,
+                                  "      <wwpn>%s</wwpn>\n", data->scsi_host.wwpn);
+                virBufferAddLit(&buf, "    </capability>\n");
+            }
+            if (data->scsi_host.flags & VIR_NODE_DEV_CAP_FLAG_HBA_VPORT_OPS) {
+                virBufferAddLit(&buf, "    <capability type='vport_ops' />\n");
+            }
+
             break;
         case VIR_NODE_DEV_CAP_SCSI:
             virBufferVSprintf(&buf, "    <host>%d</host>\n", data->scsi.host);
@@ -328,6 +372,9 @@ char *virNodeDeviceDefFormat(virConnectPtr conn,
             if (data->storage.vendor)
                 virBufferVSprintf(&buf, "    <vendor>%s</vendor>\n",
                                   data->storage.vendor);
+            if (data->storage.serial)
+                virBufferVSprintf(&buf, "    <serial>%s</serial>\n",
+                                  data->storage.serial);
             if (data->storage.flags & VIR_NODE_DEV_CAP_STORAGE_REMOVABLE) {
                 int avl = data->storage.flags &
                     VIR_NODE_DEV_CAP_STORAGE_REMOVABLE_MEDIA_AVAILABLE;
@@ -442,6 +489,7 @@ virNodeDevCapStorageParseXML(virConnectPtr conn,
     data->storage.drive_type = virXPathString(conn, "string(./drive_type[1])", ctxt);
     data->storage.model      = virXPathString(conn, "string(./model[1])", ctxt);
     data->storage.vendor     = virXPathString(conn, "string(./vendor[1])", ctxt);
+    data->storage.serial     = virXPathString(conn, "string(./serial[1])", ctxt);
 
     if ((n = virXPathNodeSet(conn, "./capability", ctxt, &nodes)) < 0) {
         virNodeDeviceReportError(conn, VIR_ERR_INTERNAL_ERROR,
@@ -561,26 +609,91 @@ virNodeDevCapScsiHostParseXML(virConnectPtr conn,
                               xmlXPathContextPtr ctxt,
                               virNodeDeviceDefPtr def,
                               xmlNodePtr node,
-                              union _virNodeDevCapData *data)
+                              union _virNodeDevCapData *data,
+                              int create)
 {
-    xmlNodePtr orignode;
-    int ret = -1;
+    xmlNodePtr orignode, *nodes = NULL;
+    int ret = -1, n = 0, i;
+    char *type = NULL;
 
     orignode = ctxt->node;
     ctxt->node = node;
 
-    if (virNodeDevCapsDefParseULong(conn, "number(./host[1])", ctxt,
+    if (create == EXISTING_DEVICE &&
+        virNodeDevCapsDefParseULong(conn, "number(./host[1])", ctxt,
                                     &data->scsi_host.host, def,
                                     _("no SCSI host ID supplied for '%s'"),
-                                    _("invalid SCSI host ID supplied for '%s'")) < 0)
+                                    _("invalid SCSI host ID supplied for '%s'")) < 0) {
         goto out;
+    }
+
+    if ((n = virXPathNodeSet(conn, "./capability", ctxt, &nodes)) < 0) {
+        virNodeDeviceReportError(conn, VIR_ERR_INTERNAL_ERROR,
+                                 _("error parsing SCSI host capabilities for '%s'"),
+                                 def->name);
+        goto out;
+    }
+
+    for (i = 0 ; i < n ; i++) {
+        type = virXMLPropString(nodes[i], "type");
+
+        if (!type) {
+            virNodeDeviceReportError(conn, VIR_ERR_INTERNAL_ERROR,
+                                     _("missing SCSI host capability type for '%s'"),
+                                     def->name);
+            goto out;
+        }
+
+        if (STREQ(type, "vport_ops")) {
+
+            data->scsi_host.flags |= VIR_NODE_DEV_CAP_FLAG_HBA_VPORT_OPS;
+
+        } else if (STREQ(type, "fc_host")) {
+
+            xmlNodePtr orignode2;
+
+            data->scsi_host.flags |= VIR_NODE_DEV_CAP_FLAG_HBA_FC_HOST;
+
+            orignode2 = ctxt->node;
+            ctxt->node = nodes[i];
+
+            if (virNodeDevCapsDefParseString(conn, "string(./wwnn[1])",
+                                             ctxt,
+                                             &data->scsi_host.wwnn,
+                                             def,
+                                             _("no WWNN supplied for '%s'")) < 0) {
+                goto out;
+            }
+
+            if (virNodeDevCapsDefParseString(conn, "string(./wwpn[1])",
+                                             ctxt,
+                                             &data->scsi_host.wwpn,
+                                             def,
+                                             _("no WWPN supplied for '%s'")) < 0) {
+                goto out;
+            }
+
+            ctxt->node = orignode2;
+
+        } else {
+            virNodeDeviceReportError(conn, VIR_ERR_INTERNAL_ERROR,
+                                     _("unknown SCSI host capability type '%s' for '%s'"),
+                                     type, def->name);
+            goto out;
+        }
+
+        VIR_FREE(type);
+    }
 
     ret = 0;
+
 out:
+    VIR_FREE(type);
     ctxt->node = orignode;
     return ret;
 }
 
+
 static int
 virNodeDevCapNetParseXML(virConnectPtr conn,
                          xmlXPathContextPtr ctxt,
@@ -848,7 +961,8 @@ static virNodeDevCapsDefPtr
 virNodeDevCapsDefParseXML(virConnectPtr conn,
                           xmlXPathContextPtr ctxt,
                           virNodeDeviceDefPtr def,
-                          xmlNodePtr node)
+                          xmlNodePtr node,
+                          int create)
 {
     virNodeDevCapsDefPtr caps;
     char *tmp;
@@ -892,7 +1006,7 @@ virNodeDevCapsDefParseXML(virConnectPtr conn,
         ret = virNodeDevCapNetParseXML(conn, ctxt, def, node, &caps->data);
         break;
     case VIR_NODE_DEV_CAP_SCSI_HOST:
-        ret = virNodeDevCapScsiHostParseXML(conn, ctxt, def, node, &caps->data);
+        ret = virNodeDevCapScsiHostParseXML(conn, ctxt, def, node, &caps->data, create);
         break;
     case VIR_NODE_DEV_CAP_SCSI:
         ret = virNodeDevCapScsiParseXML(conn, ctxt, def, node, &caps->data);
@@ -918,7 +1032,7 @@ error:
 }
 
 static virNodeDeviceDefPtr
-virNodeDeviceDefParseXML(virConnectPtr conn, xmlXPathContextPtr ctxt)
+virNodeDeviceDefParseXML(virConnectPtr conn, xmlXPathContextPtr ctxt, int create)
 {
     virNodeDeviceDefPtr def;
     virNodeDevCapsDefPtr *next_cap;
@@ -931,7 +1045,12 @@ virNodeDeviceDefParseXML(virConnectPtr conn, xmlXPathContextPtr ctxt)
     }
 
     /* Extract device name */
-    def->name = virXPathString(conn, "string(./name[1])", ctxt);
+    if (create == EXISTING_DEVICE) {
+        def->name = virXPathString(conn, "string(./name[1])", ctxt);
+    } else {
+        def->name = strdup("new device");
+    }
+
     if (!def->name) {
         virNodeDeviceReportError(conn, VIR_ERR_NO_NAME, NULL);
         goto error;
@@ -951,7 +1070,7 @@ virNodeDeviceDefParseXML(virConnectPtr conn, xmlXPathContextPtr ctxt)
 
     next_cap = &def->caps;
     for (i = 0 ; i < n ; i++) {
-        *next_cap = virNodeDevCapsDefParseXML(conn, ctxt, def, nodes[i]);
+        *next_cap = virNodeDevCapsDefParseXML(conn, ctxt, def, nodes[i], create);
         if (!*next_cap) {
             VIR_FREE(nodes);
             goto error;
@@ -969,7 +1088,7 @@ virNodeDeviceDefParseXML(virConnectPtr conn, xmlXPathContextPtr ctxt)
 }
 
 static virNodeDeviceDefPtr
-virNodeDeviceDefParseNode(virConnectPtr conn, xmlDocPtr xml, xmlNodePtr root)
+virNodeDeviceDefParseNode(virConnectPtr conn, xmlDocPtr xml, xmlNodePtr root, int create)
 {
     xmlXPathContextPtr ctxt = NULL;
     virNodeDeviceDefPtr def = NULL;
@@ -987,7 +1106,7 @@ virNodeDeviceDefParseNode(virConnectPtr conn, xmlDocPtr xml, xmlNodePtr root)
     }
 
     ctxt->node = root;
-    def = virNodeDeviceDefParseXML(conn, ctxt);
+    def = virNodeDeviceDefParseXML(conn, ctxt, create);
 
 cleanup:
     xmlXPathFreeContext(ctxt);
@@ -1015,7 +1134,7 @@ catchXMLError(void *ctx, const char *msg ATTRIBUTE_UNUSED, ...)
 }
 
 virNodeDeviceDefPtr
-virNodeDeviceDefParseString(virConnectPtr conn, const char *str)
+virNodeDeviceDefParseString(virConnectPtr conn, const char *str, int create)
 {
     xmlParserCtxtPtr pctxt;
     xmlDocPtr xml = NULL;
@@ -1046,7 +1165,7 @@ virNodeDeviceDefParseString(virConnectPtr conn, const char *str)
         goto cleanup;
     }
 
-    def = virNodeDeviceDefParseNode(conn, xml, root);
+    def = virNodeDeviceDefParseNode(conn, xml, root, create);
 
 cleanup:
     xmlFreeParserCtxt(pctxt);
@@ -1084,6 +1203,8 @@ void virNodeDevCapsDefFree(virNodeDevCapsDefPtr caps)
         VIR_FREE(data->net.address);
         break;
     case VIR_NODE_DEV_CAP_SCSI_HOST:
+        VIR_FREE(data->scsi_host.wwnn);
+        VIR_FREE(data->scsi_host.wwpn);
         break;
     case VIR_NODE_DEV_CAP_SCSI:
         VIR_FREE(data->scsi.type);
@@ -1094,6 +1215,7 @@ void virNodeDevCapsDefFree(virNodeDevCapsDefPtr caps)
         VIR_FREE(data->storage.drive_type);
         VIR_FREE(data->storage.model);
         VIR_FREE(data->storage.vendor);
+        VIR_FREE(data->storage.serial);
         break;
     case VIR_NODE_DEV_CAP_LAST:
         /* This case is here to shutup the compiler */
