@@ -36,6 +36,7 @@
 #include <fcntl.h>
 #include <stdint.h>
 #include <sys/stat.h>
+#include <sys/param.h>
 #include <dirent.h>
 
 #if HAVE_SELINUX
@@ -45,6 +46,7 @@
 #include "virterror_internal.h"
 #include "util.h"
 #include "memory.h"
+#include "node_device.h"
 
 #include "storage_backend.h"
 
@@ -54,6 +56,9 @@
 #if WITH_STORAGE_ISCSI
 #include "storage_backend_iscsi.h"
 #endif
+#if WITH_STORAGE_SCSI
+#include "storage_backend_scsi.h"
+#endif
 #if WITH_STORAGE_DISK
 #include "storage_backend_disk.h"
 #endif
@@ -61,6 +66,9 @@
 #include "storage_backend_fs.h"
 #endif
 
+#ifndef DEV_BSIZE
+#define DEV_BSIZE 512
+#endif
 
 #define VIR_FROM_THIS VIR_FROM_STORAGE
 
@@ -78,12 +86,527 @@ static virStorageBackendPtr backends[] = {
 #if WITH_STORAGE_ISCSI
     &virStorageBackendISCSI,
 #endif
+#if WITH_STORAGE_SCSI
+    &virStorageBackendSCSI,
+#endif
 #if WITH_STORAGE_DISK
     &virStorageBackendDisk,
 #endif
     NULL
 };
 
+static int track_allocation_progress = 0;
+
+enum {
+    TOOL_QEMU_IMG,
+    TOOL_KVM_IMG,
+    TOOL_QCOW_CREATE,
+};
+
+static int
+virStorageBackendCopyToFD(virConnectPtr conn,
+                          virStorageVolDefPtr vol,
+                          virStorageVolDefPtr inputvol,
+                          int fd,
+                          unsigned long long *total,
+                          int is_dest_file)
+{
+    int inputfd = -1;
+    int amtread = -1;
+    int ret = -1;
+    unsigned long long remain;
+    size_t bytes = 1024 * 1024;
+    char zerobuf[512];
+    char *buf = NULL;
+
+    if (inputvol) {
+        if ((inputfd = open(inputvol->target.path, O_RDONLY)) < 0) {
+            virReportSystemError(conn, errno,
+                                 _("could not open input path '%s'"),
+                                 inputvol->target.path);
+            goto cleanup;
+        }
+    }
+
+    bzero(&zerobuf, sizeof(zerobuf));
+
+    if (VIR_ALLOC_N(buf, bytes) < 0) {
+        virReportOOMError(conn);
+        goto cleanup;
+    }
+
+    remain = *total;
+
+    while (amtread != 0) {
+        int amtleft;
+
+        if (remain < bytes)
+            bytes = remain;
+
+        if ((amtread = saferead(inputfd, buf, bytes)) < 0) {
+            virReportSystemError(conn, errno,
+                                 _("failed reading from file '%s'"),
+                                 inputvol->target.path);
+            goto cleanup;
+        }
+        remain -= amtread;
+
+        /* Loop over amt read in 512 byte increments, looking for sparse
+         * blocks */
+        amtleft = amtread;
+        do {
+            int interval = ((512 > amtleft) ? amtleft : 512);
+            int offset = amtread - amtleft;
+
+            if (is_dest_file && memcmp(buf+offset, zerobuf, interval) == 0) {
+                if (lseek(fd, interval, SEEK_CUR) < 0) {
+                    virReportSystemError(conn, errno,
+                                         _("cannot extend file '%s'"),
+                                         vol->target.path);
+                    goto cleanup;
+                }
+            } else if (safewrite(fd, buf+offset, interval) < 0) {
+                virReportSystemError(conn, errno,
+                                     _("failed writing to file '%s'"),
+                                     vol->target.path);
+                goto cleanup;
+
+            }
+        } while ((amtleft -= 512) > 0);
+    }
+
+    if (inputfd != -1 && close(inputfd) < 0) {
+        virReportSystemError(conn, errno,
+                             _("cannot close file '%s'"),
+                             inputvol->target.path);
+        goto cleanup;
+    }
+    inputfd = -1;
+
+    *total -= remain;
+    ret = 0;
+
+cleanup:
+    if (inputfd != -1)
+        close(inputfd);
+
+    return ret;
+}
+
+static int
+virStorageBackendCreateBlockFrom(virConnectPtr conn,
+                                 virStorageVolDefPtr vol,
+                                 virStorageVolDefPtr inputvol,
+                                 unsigned int flags ATTRIBUTE_UNUSED)
+{
+    int fd = -1;
+    int ret = -1;
+    unsigned long long remain;
+
+    if ((fd = open(vol->target.path, O_RDWR)) < 0) {
+        virReportSystemError(conn, errno,
+                             _("cannot create path '%s'"),
+                             vol->target.path);
+        goto cleanup;
+    }
+
+    remain = vol->allocation;
+
+    if (inputvol) {
+        int res = virStorageBackendCopyToFD(conn, vol, inputvol,
+                                            fd, &remain, 0);
+        if (res < 0)
+            goto cleanup;
+    }
+
+    if (close(fd) < 0) {
+        virReportSystemError(conn, errno,
+                             _("cannot close file '%s'"),
+                             vol->target.path);
+        goto cleanup;
+    }
+    fd = -1;
+
+    ret = 0;
+cleanup:
+    if (fd != -1)
+        close(fd);
+
+    return ret;
+}
+
+int
+virStorageBackendCreateRaw(virConnectPtr conn,
+                           virStorageVolDefPtr vol,
+                           virStorageVolDefPtr inputvol,
+                           unsigned int flags ATTRIBUTE_UNUSED)
+{
+    int fd = -1;
+    int ret = -1;
+    unsigned long long remain;
+    char *buf = NULL;
+
+    if ((fd = open(vol->target.path, O_RDWR | O_CREAT | O_EXCL,
+                   vol->target.perms.mode)) < 0) {
+        virReportSystemError(conn, errno,
+                             _("cannot create path '%s'"),
+                             vol->target.path);
+        goto cleanup;
+    }
+
+    /* Seek to the final size, so the capacity is available upfront
+     * for progress reporting */
+    if (ftruncate(fd, vol->capacity) < 0) {
+        virReportSystemError(conn, errno,
+                             _("cannot extend file '%s'"),
+                             vol->target.path);
+        goto cleanup;
+    }
+
+    remain = vol->allocation;
+
+    if (inputvol) {
+        int res = virStorageBackendCopyToFD(conn, vol, inputvol,
+                                            fd, &remain, 1);
+        if (res < 0)
+            goto cleanup;
+    }
+
+    if (remain) {
+        if (track_allocation_progress) {
+
+            while (remain) {
+                /* Allocate in chunks of 512MiB: big-enough chunk
+                 * size and takes approx. 9s on ext3. A progress
+                 * update every 9s is a fair-enough trade-off
+                 */
+                unsigned long long bytes = 512 * 1024 * 1024;
+                int r;
+
+                if (bytes > remain)
+                    bytes = remain;
+                if ((r = safezero(fd, 0, vol->allocation - remain,
+                                  bytes)) != 0) {
+                    virReportSystemError(conn, r,
+                                         _("cannot fill file '%s'"),
+                                         vol->target.path);
+                    goto cleanup;
+                }
+                remain -= bytes;
+            }
+        } else { /* No progress bars to be shown */
+            int r;
+
+            if ((r = safezero(fd, 0, 0, remain)) != 0) {
+                virReportSystemError(conn, r,
+                                     _("cannot fill file '%s'"),
+                                     vol->target.path);
+                goto cleanup;
+            }
+        }
+    }
+
+    if (close(fd) < 0) {
+        virReportSystemError(conn, errno,
+                             _("cannot close file '%s'"),
+                             vol->target.path);
+        goto cleanup;
+    }
+    fd = -1;
+
+    ret = 0;
+cleanup:
+    if (fd != -1)
+        close(fd);
+    VIR_FREE(buf);
+
+    return ret;
+}
+
+static int
+virStorageBackendCreateQemuImg(virConnectPtr conn,
+                               virStorageVolDefPtr vol,
+                               virStorageVolDefPtr inputvol,
+                               unsigned int flags ATTRIBUTE_UNUSED)
+{
+    char size[100];
+    char *create_tool;
+    short use_kvmimg;
+
+    const char *type = virStorageVolFormatFileSystemTypeToString(vol->target.format);
+    const char *backingType = vol->backingStore.path ?
+        virStorageVolFormatFileSystemTypeToString(vol->backingStore.format) : NULL;
+
+    const char *inputBackingPath = (inputvol ? inputvol->backingStore.path
+                                             : NULL);
+    const char *inputPath = inputvol ? inputvol->target.path : NULL;
+    /* Treat input block devices as 'raw' format */
+    const char *inputType = inputPath ?
+                            virStorageVolFormatFileSystemTypeToString(inputvol->type == VIR_STORAGE_VOL_BLOCK ? VIR_STORAGE_VOL_FILE_RAW : inputvol->target.format) :
+                            NULL;
+
+    const char **imgargv;
+    const char *imgargvnormal[] = {
+        NULL, "create",
+        "-f", type,
+        vol->target.path,
+        size,
+        NULL,
+    };
+    /* Extra NULL fields are for including "backingType" when using
+     * kvm-img. It's -F backingType
+     */
+    const char *imgargvbacking[] = {
+        NULL, "create",
+        "-f", type,
+        "-b", vol->backingStore.path,
+        vol->target.path,
+        size,
+        NULL,
+        NULL,
+        NULL
+    };
+    const char *convargv[] = {
+        NULL, "convert",
+        "-f", inputType,
+        "-O", type,
+        inputPath,
+        vol->target.path,
+        NULL,
+    };
+
+    if (type == NULL) {
+        virStorageReportError(conn, VIR_ERR_INTERNAL_ERROR,
+                              _("unknown storage vol type %d"),
+                              vol->target.format);
+        return -1;
+    }
+    if (inputvol && inputType == NULL) {
+        virStorageReportError(conn, VIR_ERR_INTERNAL_ERROR,
+                              _("unknown storage vol type %d"),
+                              inputvol->target.format);
+        return -1;
+    }
+
+    if (vol->backingStore.path) {
+
+        /* XXX: Not strictly required: qemu-img has an option a different
+         * backing store, not really sure what use it serves though, and it
+         * may cause issues with lvm. Untested essentially.
+         */
+        if (inputvol &&
+            (!inputBackingPath ||
+             STRNEQ(inputBackingPath, vol->backingStore.path))) {
+            virStorageReportError(conn, VIR_ERR_INTERNAL_ERROR,
+                                  "%s", _("a different backing store can not "
+                                          "be specified."));
+            return -1;
+        }
+
+        if (backingType == NULL) {
+            virStorageReportError(conn, VIR_ERR_INTERNAL_ERROR,
+                                  _("unknown storage vol backing store type %d"),
+                                  vol->backingStore.format);
+            return -1;
+        }
+        if (access(vol->backingStore.path, R_OK) != 0) {
+            virReportSystemError(conn, errno,
+                                 _("inaccessible backing store volume %s"),
+                                 vol->backingStore.path);
+            return -1;
+        }
+    }
+
+    if ((create_tool = virFindFileInPath("kvm-img")) != NULL)
+        use_kvmimg = 1;
+    else if ((create_tool = virFindFileInPath("qemu-img")) != NULL)
+        use_kvmimg = 0;
+    else {
+        virStorageReportError(conn, VIR_ERR_INTERNAL_ERROR,
+                              "%s", _("unable to find kvm-img or qemu-img"));
+        return -1;
+    }
+
+    if (inputvol) {
+        convargv[0] = create_tool;
+        imgargv = convargv;
+    } else if (vol->backingStore.path) {
+        imgargvbacking[0] = create_tool;
+        if (use_kvmimg) {
+            imgargvbacking[6] = "-F";
+            imgargvbacking[7] = backingType;
+            imgargvbacking[8] = vol->target.path;
+            imgargvbacking[9] = size;
+        }
+        imgargv = imgargvbacking;
+    } else {
+        imgargvnormal[0] = create_tool;
+        imgargv = imgargvnormal;
+    }
+
+
+    /* Size in KB */
+    snprintf(size, sizeof(size), "%lluK", vol->capacity/1024);
+
+    if (virRun(conn, imgargv, NULL) < 0) {
+        VIR_FREE(imgargv[0]);
+        return -1;
+    }
+
+    VIR_FREE(imgargv[0]);
+
+    return 0;
+}
+
+/*
+ * Xen removed the fully-functional qemu-img, and replaced it
+ * with a partially functional qcow-create. Go figure ??!?
+ */
+static int
+virStorageBackendCreateQcowCreate(virConnectPtr conn,
+                                  virStorageVolDefPtr vol,
+                                  virStorageVolDefPtr inputvol,
+                                  unsigned int flags ATTRIBUTE_UNUSED)
+{
+    char size[100];
+    const char *imgargv[4];
+
+    if (inputvol) {
+        virStorageReportError(conn, VIR_ERR_INTERNAL_ERROR, "%s",
+                              _("cannot copy from volume with qcow-create"));
+        return -1;
+    }
+
+    if (vol->target.format != VIR_STORAGE_VOL_FILE_QCOW2) {
+        virStorageReportError(conn, VIR_ERR_INTERNAL_ERROR,
+                              _("unsupported storage vol type %d"),
+                              vol->target.format);
+        return -1;
+    }
+    if (vol->backingStore.path != NULL) {
+        virStorageReportError(conn, VIR_ERR_NO_SUPPORT, "%s",
+                              _("copy-on-write image not supported with "
+                                      "qcow-create"));
+        return -1;
+    }
+
+    /* Size in MB - yes different units to qemu-img :-( */
+    snprintf(size, sizeof(size), "%llu", vol->capacity/1024/1024);
+
+    imgargv[0] = virFindFileInPath("qcow-create");
+    imgargv[1] = size;
+    imgargv[2] = vol->target.path;
+    imgargv[3] = NULL;
+
+    if (virRun(conn, imgargv, NULL) < 0) {
+        VIR_FREE(imgargv[0]);
+        return -1;
+    }
+
+    VIR_FREE(imgargv[0]);
+
+    return 0;
+}
+
+virStorageBackendBuildVolFrom
+virStorageBackendFSImageToolTypeToFunc(virConnectPtr conn, int tool_type)
+{
+    switch (tool_type) {
+    case TOOL_KVM_IMG:
+    case TOOL_QEMU_IMG:
+        return virStorageBackendCreateQemuImg;
+    case TOOL_QCOW_CREATE:
+        return virStorageBackendCreateQcowCreate;
+    default:
+        virStorageReportError(conn, VIR_ERR_INTERNAL_ERROR,
+                              _("Unknown file create tool type '%d'."),
+                              tool_type);
+    }
+
+    return NULL;
+}
+
+int
+virStorageBackendFindFSImageTool(char **tool)
+{
+    int tool_type = -1;
+    char *tmp = NULL;
+
+    if ((tmp = virFindFileInPath("kvm-img")) != NULL) {
+        tool_type = TOOL_KVM_IMG;
+    } else if ((tmp = virFindFileInPath("qemu-img")) != NULL) {
+        tool_type = TOOL_QEMU_IMG;
+    } else if ((tmp = virFindFileInPath("qcow-create")) != NULL) {
+        tool_type = TOOL_QCOW_CREATE;
+    }
+
+    if (tool)
+        *tool = tmp;
+    else
+        VIR_FREE(tmp);
+
+    return tool_type;
+}
+
+virStorageBackendBuildVolFrom
+virStorageBackendGetBuildVolFromFunction(virConnectPtr conn,
+                                         virStorageVolDefPtr vol,
+                                         virStorageVolDefPtr inputvol)
+{
+    int tool_type;
+
+    if (!inputvol)
+        return NULL;
+
+    /* If either volume is a non-raw file vol, we need to use an external
+     * tool for converting
+     */
+    if ((vol->type == VIR_STORAGE_VOL_FILE &&
+         vol->target.format != VIR_STORAGE_VOL_FILE_RAW) ||
+        (inputvol->type == VIR_STORAGE_VOL_FILE &&
+         inputvol->target.format != VIR_STORAGE_VOL_FILE_RAW)) {
+
+        if ((tool_type = virStorageBackendFindFSImageTool(NULL)) < 0) {
+            virStorageReportError(conn, VIR_ERR_INTERNAL_ERROR,
+                                  "%s", _("creation of non-raw file images is "
+                                          "not supported without qemu-img."));
+            return NULL;
+        }
+
+        return virStorageBackendFSImageToolTypeToFunc(conn, tool_type);
+    }
+
+    if (vol->type == VIR_STORAGE_VOL_BLOCK)
+        return virStorageBackendCreateBlockFrom;
+    else
+        return virStorageBackendCreateRaw;
+}
+
+#if defined(UDEVADM) || defined(UDEVSETTLE)
+void virWaitForDevices(virConnectPtr conn)
+{
+#ifdef UDEVADM
+    const char *const settleprog[] = { UDEVADM, "settle", NULL };
+#else
+    const char *const settleprog[] = { UDEVSETTLE, NULL };
+#endif
+    int exitstatus;
+
+    if (access(settleprog[0], X_OK) != 0)
+        return;
+
+    /*
+     * NOTE: we ignore errors here; this is just to make sure that any device
+     * nodes that are being created finish before we try to scan them.
+     * If this fails for any reason, we still have the backup of polling for
+     * 5 seconds for device nodes.
+     */
+    virRun(conn, settleprog, &exitstatus);
+}
+#else
+void virWaitForDevices(virConnectPtr conn ATTRIBUTE_UNUSED) {}
+#endif
+
 
 virStorageBackendPtr
 virStorageBackendForType(int type) {
@@ -146,37 +669,17 @@ virStorageBackendUpdateVolInfo(virConnectPtr conn,
     return 0;
 }
 
-struct diskType {
-    int part_table_type;
-    unsigned short offset;
-    unsigned short length;
-    unsigned long long magic;
-};
-
-static struct diskType const disk_types[] = {
-    { VIR_STORAGE_POOL_DISK_LVM2, 0x218, 8, 0x31303020324D564CULL },
-    { VIR_STORAGE_POOL_DISK_GPT,  0x200, 8, 0x5452415020494645ULL },
-    { VIR_STORAGE_POOL_DISK_DVH,  0x0,   4, 0x41A9E50BULL },
-    { VIR_STORAGE_POOL_DISK_MAC,  0x0,   2, 0x5245ULL },
-    { VIR_STORAGE_POOL_DISK_BSD,  0x40,  4, 0x82564557ULL },
-    { VIR_STORAGE_POOL_DISK_SUN,  0x1fc, 2, 0xBEDAULL },
-    /*
-     * NOTE: pc98 is funky; the actual signature is 0x55AA (just like dos), so
-     * we can't use that.  At the moment I'm relying on the "dummy" IPL
-     * bootloader data that comes from parted.  Luckily, the chances of running
-     * into a pc98 machine running libvirt are approximately nil.
-     */
-    /*{ 0x1fe, 2, 0xAA55UL },*/
-    { VIR_STORAGE_POOL_DISK_PC98, 0x0,   8, 0x314C5049000000CBULL },
-    /*
-     * NOTE: the order is important here; some other disk types (like GPT and
-     * and PC98) also have 0x55AA at this offset.  For that reason, the DOS
-     * one must be the last one.
-     */
-    { VIR_STORAGE_POOL_DISK_DOS,  0x1fe, 2, 0xAA55ULL },
-    { -1,                         0x0,   0, 0x0ULL },
-};
-
+/*
+ * virStorageBackendUpdateVolTargetInfoFD:
+ * @conn: connection to report errors on
+ * @target: target definition ptr of volume to update
+ * @fd: fd of storage volume to update
+ * @allocation: If not NULL, updated allocation information will be stored
+ * @capacity: If not NULL, updated capacity info will be stored
+ *
+ * Returns 0 for success-1 on a legitimate error condition,
+ *    -2 if passed FD isn't a regular, char, or block file.
+ */
 int
 virStorageBackendUpdateVolTargetInfoFD(virConnectPtr conn,
                                        virStorageVolTargetPtr target,
@@ -205,7 +708,7 @@ virStorageBackendUpdateVolTargetInfoFD(virConnectPtr conn,
         if (S_ISREG(sb.st_mode)) {
 #ifndef __MINGW32__
             *allocation = (unsigned long long)sb.st_blocks *
-                (unsigned long long)sb.st_blksize;
+                          (unsigned long long)DEV_BSIZE;
 #else
             *allocation = sb.st_size;
 #endif
@@ -234,41 +737,6 @@ virStorageBackendUpdateVolTargetInfoFD(virConnectPtr conn,
         }
     }
 
-    /* make sure to set the target format "unknown" to begin with */
-    target->format = VIR_STORAGE_POOL_DISK_UNKNOWN;
-
-    if (S_ISBLK(sb.st_mode)) {
-        off_t start;
-        int i;
-        unsigned char buffer[1024];
-        ssize_t bytes;
-
-        start = lseek(fd, 0, SEEK_SET);
-        if (start < 0) {
-            virReportSystemError(conn, errno,
-                                 _("cannot seek to beginning of file '%s'"),
-                                 target->path);
-            return -1;
-        }
-        bytes = saferead(fd, buffer, sizeof(buffer));
-        if (bytes < 0) {
-            virReportSystemError(conn, errno,
-                                 _("cannot read beginning of file '%s'"),
-                                 target->path);
-            return -1;
-        }
-
-        for (i = 0; disk_types[i].part_table_type != -1; i++) {
-            if (disk_types[i].offset + disk_types[i].length > bytes)
-                continue;
-            if (memcmp(buffer+disk_types[i].offset, &disk_types[i].magic,
-                disk_types[i].length) == 0) {
-                target->format = disk_types[i].part_table_type;
-                break;
-            }
-        }
-    }
-
     target->perms.mode = sb.st_mode & S_IRWXUGO;
     target->perms.uid = sb.st_uid;
     target->perms.gid = sb.st_gid;
@@ -301,30 +769,11 @@ virStorageBackendUpdateVolTargetInfoFD(virConnectPtr conn,
     return 0;
 }
 
-#if defined(UDEVADM) || defined(UDEVSETTLE)
 void virStorageBackendWaitForDevices(virConnectPtr conn)
 {
-#ifdef UDEVADM
-    const char *const settleprog[] = { UDEVADM, "settle", NULL };
-#else
-    const char *const settleprog[] = { UDEVSETTLE, NULL };
-#endif
-    int exitstatus;
-
-    if (access(settleprog[0], X_OK) != 0)
-        return;
-
-    /*
-     * NOTE: we ignore errors here; this is just to make sure that any device
-     * nodes that are being created finish before we try to scan them.
-     * If this fails for any reason, we still have the backup of polling for
-     * 5 seconds for device nodes.
-     */
-    virRun(conn, settleprog, &exitstatus);
+    virWaitForDevices(conn);
+    return;
 }
-#else
-void virStorageBackendWaitForDevices(virConnectPtr conn ATTRIBUTE_UNUSED) {}
-#endif
 
 /*
  * Given a volume path directly in /dev/XXX, iterate over the
@@ -387,17 +836,14 @@ virStorageBackendStablePath(virConnectPtr conn,
         if (dent->d_name[0] == '.')
             continue;
 
-        if (VIR_ALLOC_N(stablepath, strlen(pool->def->target.path) +
-                        1 + strlen(dent->d_name) + 1) < 0) {
+        if (virAsprintf(&stablepath, "%s/%s",
+                        pool->def->target.path,
+                        dent->d_name) == -1) {
             virReportOOMError(conn);
             closedir(dh);
             return NULL;
         }
 
-        strcpy(stablepath, pool->def->target.path);
-        strcat(stablepath, "/");
-        strcat(stablepath, dent->d_name);
-
         if (virFileLinkPointsTo(stablepath, devpath)) {
             closedir(dh);
             return stablepath;
