@@ -594,14 +594,13 @@ virStorageBackendLogicalCreateVol(virConnectPtr conn,
         /* A target path passed to CreateVol has no meaning */
         VIR_FREE(vol->target.path);
     }
-    if (VIR_ALLOC_N(vol->target.path, strlen(pool->def->target.path) +
-                    1 + strlen(vol->name) + 1) < 0) {
+
+    if (virAsprintf(&vol->target.path, "%s/%s",
+                    pool->def->target.path,
+                    vol->name) == -1) {
         virReportOOMError(conn);
         return -1;
     }
-    strcpy(vol->target.path, pool->def->target.path);
-    strcat(vol->target.path, "/");
-    strcat(vol->target.path, vol->name);
 
     if (virRun(conn, cmdargv, NULL) < 0)
         return -1;
@@ -654,6 +653,21 @@ virStorageBackendLogicalCreateVol(virConnectPtr conn,
     return -1;
 }
 
+static int
+virStorageBackendLogicalBuildVolFrom(virConnectPtr conn,
+                                     virStorageVolDefPtr vol,
+                                     virStorageVolDefPtr inputvol,
+                                     unsigned int flags)
+{
+    virStorageBackendBuildVolFrom build_func;
+
+    build_func = virStorageBackendGetBuildVolFromFunction(conn, vol, inputvol);
+    if (!build_func)
+        return -1;
+
+    return build_func(conn, vol, inputvol, flags);
+}
+
 static int
 virStorageBackendLogicalDeleteVol(virConnectPtr conn,
                                   virStoragePoolObjPtr pool ATTRIBUTE_UNUSED,
@@ -680,6 +694,8 @@ virStorageBackend virStorageBackendLogical = {
     .refreshPool = virStorageBackendLogicalRefreshPool,
     .stopPool = virStorageBackendLogicalStopPool,
     .deletePool = virStorageBackendLogicalDeletePool,
+    .buildVol = NULL,
+    .buildVolFrom = virStorageBackendLogicalBuildVolFrom,
     .createVol = virStorageBackendLogicalCreateVol,
     .deleteVol = virStorageBackendLogicalDeleteVol,
 };
