@@ -45,12 +45,9 @@
 
 #define VIR_FROM_THIS VIR_FROM_STORAGE
 
-/* Work around broken limits.h on debian etch */
-#if defined __GNUC__ && defined _GCC_LIMITS_H_ && ! defined ULLONG_MAX
-# define ULLONG_MAX   ULONG_LONG_MAX
-#endif
-
-#define virStorageLog(msg...) fprintf(stderr, msg)
+#define virStorageError(conn, code, fmt...)                             \
+            virReportErrorHelper(conn, VIR_FROM_STORAGE, code, __FILE__,\
+                                  __FUNCTION__, __LINE__, fmt)
 
 VIR_ENUM_IMPL(virStoragePool,
               VIR_STORAGE_POOL_LAST,
@@ -66,7 +63,7 @@ VIR_ENUM_IMPL(virStoragePoolFormatFileSystem,
 
 VIR_ENUM_IMPL(virStoragePoolFormatFileSystemNet,
               VIR_STORAGE_POOL_NETFS_LAST,
-              "auto", "nfs")
+              "auto", "nfs", "glusterfs")
 
 VIR_ENUM_IMPL(virStoragePoolFormatDisk,
               VIR_STORAGE_POOL_DISK_LAST,
@@ -91,6 +88,12 @@ VIR_ENUM_IMPL(virStorageVolFormatFileSystem,
               "cloop", "cow", "dmg", "iso",
               "qcow", "qcow2", "vmdk", "vpc")
 
+VIR_ENUM_IMPL(virStoragePartedFsType,
+              VIR_STORAGE_PARTED_FS_TYPE_LAST,
+              "ext2", "ext2", "fat16",
+              "fat32", "linux-swap",
+              "ext2", "ext2",
+              "extended")
 
 typedef const char *(*virStorageVolFormatToString)(int format);
 typedef int (*virStorageVolFormatFromString)(const char *format);
@@ -187,6 +190,14 @@ static virStoragePoolTypeInfo poolTypeInfo[] = {
             .formatToString = virStoragePoolFormatDiskTypeToString,
         }
     },
+    { .poolType = VIR_STORAGE_POOL_SCSI,
+      .poolOptions = {
+            .flags = (VIR_STORAGE_POOL_SOURCE_ADAPTER),
+        },
+      .volOptions = {
+            .formatToString = virStoragePoolFormatDiskTypeToString,
+        }
+    },
     { .poolType = VIR_STORAGE_POOL_DISK,
       .poolOptions = {
             .flags = (VIR_STORAGE_POOL_SOURCE_DEVICE),
@@ -269,6 +280,7 @@ virStoragePoolSourceFree(virStoragePoolSourcePtr source) {
     VIR_FREE(source->devices);
     VIR_FREE(source->dir);
     VIR_FREE(source->name);
+    VIR_FREE(source->adapter);
 
     if (source->authType == VIR_STORAGE_POOL_AUTH_CHAP) {
         VIR_FREE(source->auth.chap.login);
@@ -352,14 +364,14 @@ static int
 virStoragePoolDefParseAuthChap(virConnectPtr conn,
                                xmlXPathContextPtr ctxt,
                                virStoragePoolAuthChapPtr auth) {
-    auth->login = virXPathString(conn, "string(/pool/source/auth/@login)", ctxt);
+    auth->login = virXPathString(conn, "string(./source/auth/@login)", ctxt);
     if (auth->login == NULL) {
         virStorageReportError(conn, VIR_ERR_XML_ERROR,
                               "%s", _("missing auth host attribute"));
         return -1;
     }
 
-    auth->passwd = virXPathString(conn, "string(/pool/source/auth/@passwd)", ctxt);
+    auth->passwd = virXPathString(conn, "string(./source/auth/@passwd)", ctxt);
     if (auth->passwd == NULL) {
         virStorageReportError(conn, VIR_ERR_XML_ERROR,
                               "%s", _("missing auth passwd attribute"));
@@ -371,62 +383,82 @@ virStoragePoolDefParseAuthChap(virConnectPtr conn,
 
 
 static int
-virStoragePoolDefParsePerms(virConnectPtr conn,
-                            xmlXPathContextPtr ctxt,
-                            virStoragePermsPtr perms) {
+virStorageDefParsePerms(virConnectPtr conn,
+                        xmlXPathContextPtr ctxt,
+                        virStoragePermsPtr perms,
+                        const char *permxpath,
+                        int defaultmode) {
     char *mode;
     long v;
+    int ret = -1;
+    xmlNodePtr relnode;
+    xmlNodePtr node;
 
-    mode = virXPathString(conn, "string(/pool/permissions/mode)", ctxt);
+    node = virXPathNode(conn, permxpath, ctxt);
+    if (node == NULL) {
+        /* Set default values if there is not <permissions> element */
+        perms->mode = defaultmode;
+        perms->uid = getuid();
+        perms->gid = getgid();
+        perms->label = NULL;
+        return 0;
+    }
+
+    relnode = ctxt->node;
+    ctxt->node = node;
+
+    mode = virXPathString(conn, "string(./mode)", ctxt);
     if (!mode) {
-        perms->mode = 0700;
+        perms->mode = defaultmode;
     } else {
         char *end = NULL;
         perms->mode = strtol(mode, &end, 8);
-        VIR_FREE(mode);
         if (*end || perms->mode < 0 || perms->mode > 0777) {
+            VIR_FREE(mode);
             virStorageReportError(conn, VIR_ERR_XML_ERROR,
                                   "%s", _("malformed octal mode"));
-            return -1;
+            goto error;
         }
+        VIR_FREE(mode);
     }
 
-    if (virXPathNode(conn, "/pool/permissions/owner", ctxt) == NULL) {
+    if (virXPathNode(conn, "./owner", ctxt) == NULL) {
         perms->uid = getuid();
     } else {
-        if (virXPathLong(conn, "number(/pool/permissions/owner)", ctxt, &v) < 0) {
+        if (virXPathLong(conn, "number(./owner)", ctxt, &v) < 0) {
             virStorageReportError(conn, VIR_ERR_XML_ERROR,
                                   "%s", _("malformed owner element"));
-            return -1;
+            goto error;
         }
         perms->uid = (int)v;
     }
 
-    if (virXPathNode(conn, "/pool/permissions/group", ctxt) == NULL) {
+    if (virXPathNode(conn, "./group", ctxt) == NULL) {
         perms->gid = getgid();
     } else {
-        if (virXPathLong(conn, "number(/pool/permissions/group)", ctxt, &v) < 0) {
+        if (virXPathLong(conn, "number(./group)", ctxt, &v) < 0) {
             virStorageReportError(conn, VIR_ERR_XML_ERROR,
                                   "%s", _("malformed group element"));
-            return -1;
+            goto error;
         }
         perms->gid = (int)v;
     }
 
     /* NB, we're ignoring missing labels here - they'll simply inherit */
-    perms->label = virXPathString(conn, "string(/pool/permissions/label)", ctxt);
+    perms->label = virXPathString(conn, "string(./label)", ctxt);
 
-    return 0;
+    ret = 0;
+error:
+    ctxt->node = relnode;
+    return ret;
 }
 
-
 static virStoragePoolDefPtr
-virStoragePoolDefParseDoc(virConnectPtr conn,
-                          xmlXPathContextPtr ctxt,
-                          xmlNodePtr root) {
+virStoragePoolDefParseXML(virConnectPtr conn,
+                          xmlXPathContextPtr ctxt) {
     virStoragePoolOptionsPtr options;
     virStoragePoolDefPtr ret;
-    xmlChar *type = NULL;
+    char *type = NULL;
     char *uuid = NULL;
     char *authType = NULL;
 
@@ -435,13 +467,7 @@ virStoragePoolDefParseDoc(virConnectPtr conn,
         return NULL;
     }
 
-    if (STRNEQ((const char *)root->name, "pool")) {
-        virStorageReportError(conn, VIR_ERR_XML_ERROR,
-                          "%s", _("unknown root element for storage pool"));
-        goto cleanup;
-    }
-
-    type = xmlGetProp(root, BAD_CAST "type");
+    type = virXPathString(conn, "string(./@type)", ctxt);
     if ((ret->type = virStoragePoolTypeFromString((const char *)type)) < 0) {
         virStorageReportError(conn, VIR_ERR_INTERNAL_ERROR,
                               _("unknown storage pool type %s"), (const char*)type);
@@ -455,17 +481,17 @@ virStoragePoolDefParseDoc(virConnectPtr conn,
         goto cleanup;
     }
 
-    ret->name = virXPathString(conn, "string(/pool/name)", ctxt);
+    ret->name = virXPathString(conn, "string(./name)", ctxt);
     if (ret->name == NULL &&
         options->flags & VIR_STORAGE_POOL_SOURCE_NAME)
-        ret->name = virXPathString(conn, "string(/pool/source/name)", ctxt);
+        ret->name = virXPathString(conn, "string(./source/name)", ctxt);
     if (ret->name == NULL) {
         virStorageReportError(conn, VIR_ERR_XML_ERROR,
                               "%s", _("missing pool source name element"));
         goto cleanup;
     }
 
-    uuid = virXPathString(conn, "string(/pool/uuid)", ctxt);
+    uuid = virXPathString(conn, "string(./uuid)", ctxt);
     if (uuid == NULL) {
         if (virUUIDGenerate(ret->uuid) < 0) {
             virStorageReportError(conn, VIR_ERR_INTERNAL_ERROR,
@@ -482,7 +508,7 @@ virStoragePoolDefParseDoc(virConnectPtr conn,
     }
 
     if (options->formatFromString) {
-        char *format = virXPathString(conn, "string(/pool/source/format/@type)", ctxt);
+        char *format = virXPathString(conn, "string(./source/format/@type)", ctxt);
         if (format == NULL)
             ret->source.format = options->defaultFormat;
         else
@@ -498,7 +524,7 @@ virStoragePoolDefParseDoc(virConnectPtr conn,
     }
 
     if (options->flags & VIR_STORAGE_POOL_SOURCE_HOST) {
-        if ((ret->source.host.name = virXPathString(conn, "string(/pool/source/host/@name)", ctxt)) == NULL) {
+        if ((ret->source.host.name = virXPathString(conn, "string(./source/host/@name)", ctxt)) == NULL) {
             virStorageReportError(conn, VIR_ERR_XML_ERROR,
                              "%s", _("missing storage pool source host name"));
             goto cleanup;
@@ -508,7 +534,7 @@ virStoragePoolDefParseDoc(virConnectPtr conn,
         xmlNodePtr *nodeset = NULL;
         int nsource, i;
 
-        if ((nsource = virXPathNodeSet(conn, "/pool/source/device", ctxt, &nodeset)) < 0) {
+        if ((nsource = virXPathNodeSet(conn, "./source/device", ctxt, &nodeset)) < 0) {
             virStorageReportError(conn, VIR_ERR_XML_ERROR,
                         "%s", _("cannot extract storage pool source devices"));
             goto cleanup;
@@ -532,14 +558,14 @@ virStoragePoolDefParseDoc(virConnectPtr conn,
         ret->source.ndevice = nsource;
     }
     if (options->flags & VIR_STORAGE_POOL_SOURCE_DIR) {
-        if ((ret->source.dir = virXPathString(conn, "string(/pool/source/dir/@path)", ctxt)) == NULL) {
+        if ((ret->source.dir = virXPathString(conn, "string(./source/dir/@path)", ctxt)) == NULL) {
             virStorageReportError(conn, VIR_ERR_XML_ERROR,
                                 "%s", _("missing storage pool source path"));
             goto cleanup;
         }
     }
     if (options->flags & VIR_STORAGE_POOL_SOURCE_NAME) {
-        ret->source.name = virXPathString(conn, "string(/pool/source/name)",
+        ret->source.name = virXPathString(conn, "string(./source/name)",
                                           ctxt);
         if (ret->source.name == NULL) {
             /* source name defaults to pool name */
@@ -551,8 +577,17 @@ virStoragePoolDefParseDoc(virConnectPtr conn,
         }
     }
 
+    if (options->flags & VIR_STORAGE_POOL_SOURCE_ADAPTER) {
+        if ((ret->source.adapter = virXPathString(conn,
+                                                  "string(./source/adapter/@name)",
+                                                  ctxt)) == NULL) {
+            virStorageReportError(conn, VIR_ERR_XML_ERROR,
+                             "%s", _("missing storage pool source adapter name"));
+            goto cleanup;
+        }
+    }
 
-    authType = virXPathString(conn, "string(/pool/source/auth/@type)", ctxt);
+    authType = virXPathString(conn, "string(./source/auth/@type)", ctxt);
     if (authType == NULL) {
         ret->source.authType = VIR_STORAGE_POOL_AUTH_NONE;
     } else {
@@ -573,13 +608,14 @@ virStoragePoolDefParseDoc(virConnectPtr conn,
             goto cleanup;
     }
 
-    if ((ret->target.path = virXPathString(conn, "string(/pool/target/path)", ctxt)) == NULL) {
+    if ((ret->target.path = virXPathString(conn, "string(./target/path)", ctxt)) == NULL) {
         virStorageReportError(conn, VIR_ERR_XML_ERROR,
                               "%s", _("missing storage pool target path"));
         goto cleanup;
     }
 
-    if (virStoragePoolDefParsePerms(conn, ctxt, &ret->target.perms) < 0)
+    if (virStorageDefParsePerms(conn, ctxt, &ret->target.perms,
+                                "./target/permissions", 0700) < 0)
         goto cleanup;
 
     return ret;
@@ -613,6 +649,32 @@ catchXMLError (void *ctx, const char *msg ATTRIBUTE_UNUSED, ...)
 }
 
 virStoragePoolDefPtr
+virStoragePoolDefParseNode(virConnectPtr conn,
+                           xmlDocPtr xml,
+                           xmlNodePtr root) {
+    xmlXPathContextPtr ctxt = NULL;
+    virStoragePoolDefPtr def = NULL;
+
+    if (STRNEQ((const char *)root->name, "pool")) {
+        virStorageReportError(conn, VIR_ERR_XML_ERROR,
+                          "%s", _("unknown root element for storage pool"));
+        goto cleanup;
+    }
+
+    ctxt = xmlXPathNewContext(xml);
+    if (ctxt == NULL) {
+        virReportOOMError(conn);
+        goto cleanup;
+    }
+
+    ctxt->node = root;
+    def = virStoragePoolDefParseXML(conn, ctxt);
+cleanup:
+    xmlXPathFreeContext(ctxt);
+    return def;
+}
+
+static virStoragePoolDefPtr
 virStoragePoolDefParse(virConnectPtr conn,
                        const char *xmlStr,
                        const char *filename) {
@@ -620,7 +682,6 @@ virStoragePoolDefParse(virConnectPtr conn,
     xmlParserCtxtPtr pctxt;
     xmlDocPtr xml = NULL;
     xmlNodePtr node = NULL;
-    xmlXPathContextPtr ctxt = NULL;
 
     /* Set up a parser context so we can catch the details of XML errors. */
     pctxt = xmlNewParserCtxt ();
@@ -630,10 +691,17 @@ virStoragePoolDefParse(virConnectPtr conn,
     pctxt->_private = conn;
 
     if (conn) virResetError (&conn->err);
-    xml = xmlCtxtReadDoc (pctxt, BAD_CAST xmlStr,
-                          filename ? filename : "storage.xml", NULL,
-                          XML_PARSE_NOENT | XML_PARSE_NONET |
-                          XML_PARSE_NOWARNING);
+    if (filename) {
+        xml = xmlCtxtReadFile (pctxt, filename, NULL,
+                               XML_PARSE_NOENT | XML_PARSE_NONET |
+                               XML_PARSE_NOWARNING);
+    } else {
+        xml = xmlCtxtReadDoc (pctxt, BAD_CAST xmlStr,
+                              "storage.xml", NULL,
+                              XML_PARSE_NOENT | XML_PARSE_NONET |
+                              XML_PARSE_NOWARNING);
+    }
+
     if (!xml) {
         if (conn && conn->err.code == VIR_ERR_NONE)
               virStorageReportError(conn, VIR_ERR_XML_ERROR,
@@ -641,12 +709,6 @@ virStoragePoolDefParse(virConnectPtr conn,
         goto cleanup;
     }
 
-    ctxt = xmlXPathNewContext(xml);
-    if (ctxt == NULL) {
-        virReportOOMError(conn);
-        goto cleanup;
-    }
-
     node = xmlDocGetRootElement(xml);
     if (node == NULL) {
         virStorageReportError(conn, VIR_ERR_XML_ERROR,
@@ -654,21 +716,33 @@ virStoragePoolDefParse(virConnectPtr conn,
         goto cleanup;
     }
 
-    ret = virStoragePoolDefParseDoc(conn, ctxt, node);
+    ret = virStoragePoolDefParseNode(conn, xml, node);
 
     xmlFreeParserCtxt (pctxt);
-    xmlXPathFreeContext(ctxt);
     xmlFreeDoc(xml);
 
     return ret;
 
  cleanup:
     xmlFreeParserCtxt (pctxt);
-    xmlXPathFreeContext(ctxt);
     xmlFreeDoc(xml);
     return NULL;
 }
 
+virStoragePoolDefPtr
+virStoragePoolDefParseString(virConnectPtr conn,
+                             const char *xmlStr)
+{
+    return virStoragePoolDefParse(conn, xmlStr, NULL);
+}
+
+virStoragePoolDefPtr
+virStoragePoolDefParseFile(virConnectPtr conn,
+                           const char *filename)
+{
+    return virStoragePoolDefParse(conn, NULL, filename);
+}
+
 static int
 virStoragePoolSourceFormat(virConnectPtr conn,
                            virBufferPtr buf,
@@ -800,55 +874,6 @@ virStoragePoolDefFormat(virConnectPtr conn,
 }
 
 
-static int
-virStorageVolDefParsePerms(virConnectPtr conn,
-                           xmlXPathContextPtr ctxt,
-                           virStoragePermsPtr perms) {
-    char *mode;
-    long v;
-
-    mode = virXPathString(conn, "string(/volume/permissions/mode)", ctxt);
-    if (!mode) {
-        perms->mode = 0600;
-    } else {
-        char *end = NULL;
-        perms->mode = strtol(mode, &end, 8);
-        VIR_FREE(mode);
-        if (*end || perms->mode < 0 || perms->mode > 0777) {
-            virStorageReportError(conn, VIR_ERR_XML_ERROR,
-                                  "%s", _("malformed octal mode"));
-            return -1;
-        }
-    }
-
-    if (virXPathNode(conn, "/volume/permissions/owner", ctxt) == NULL) {
-        perms->uid = getuid();
-    } else {
-        if (virXPathLong(conn, "number(/volume/permissions/owner)", ctxt, &v) < 0) {
-            virStorageReportError(conn, VIR_ERR_XML_ERROR,
-                                  "%s", _("missing owner element"));
-            return -1;
-        }
-        perms->uid = (int)v;
-    }
-    if (virXPathNode(conn, "/volume/permissions/group", ctxt) == NULL) {
-        perms->gid = getgid();
-    } else {
-        if (virXPathLong(conn, "number(/volume/permissions/group)", ctxt, &v) < 0) {
-            virStorageReportError(conn, VIR_ERR_XML_ERROR,
-                                  "%s", _("missing group element"));
-            return -1;
-        }
-        perms->gid = (int)v;
-    }
-
-    /* NB, we're ignoring missing labels here - they'll simply inherit */
-    perms->label = virXPathString(conn, "string(/volume/permissions/label)", ctxt);
-
-    return 0;
-}
-
-
 static int
 virStorageSize(virConnectPtr conn,
                const char *unit,
@@ -922,10 +947,9 @@ virStorageSize(virConnectPtr conn,
 }
 
 static virStorageVolDefPtr
-virStorageVolDefParseDoc(virConnectPtr conn,
+virStorageVolDefParseXML(virConnectPtr conn,
                          virStoragePoolDefPtr pool,
-                         xmlXPathContextPtr ctxt,
-                         xmlNodePtr root) {
+                         xmlXPathContextPtr ctxt) {
     virStorageVolDefPtr ret;
     virStorageVolOptionsPtr options;
     char *allocation = NULL;
@@ -941,13 +965,7 @@ virStorageVolDefParseDoc(virConnectPtr conn,
         return NULL;
     }
 
-    if (STRNEQ((const char *)root->name, "volume")) {
-        virStorageReportError(conn, VIR_ERR_XML_ERROR,
-                              "%s", _("unknown root element"));
-        goto cleanup;
-    }
-
-    ret->name = virXPathString(conn, "string(/volume/name)", ctxt);
+    ret->name = virXPathString(conn, "string(./name)", ctxt);
     if (ret->name == NULL) {
         virStorageReportError(conn, VIR_ERR_XML_ERROR,
                               "%s", _("missing volume name element"));
@@ -955,10 +973,10 @@ virStorageVolDefParseDoc(virConnectPtr conn,
     }
 
     /* Auto-generated so deliberately ignore */
-    /*ret->key = virXPathString(conn, "string(/volume/key)", ctxt);*/
+    /*ret->key = virXPathString(conn, "string(./key)", ctxt);*/
 
-    capacity = virXPathString(conn, "string(/volume/capacity)", ctxt);
-    unit = virXPathString(conn, "string(/volume/capacity/@unit)", ctxt);
+    capacity = virXPathString(conn, "string(./capacity)", ctxt);
+    unit = virXPathString(conn, "string(./capacity/@unit)", ctxt);
     if (capacity == NULL) {
         virStorageReportError(conn, VIR_ERR_XML_ERROR,
                               "%s", _("missing capacity element"));
@@ -969,9 +987,9 @@ virStorageVolDefParseDoc(virConnectPtr conn,
     VIR_FREE(capacity);
     VIR_FREE(unit);
 
-    allocation = virXPathString(conn, "string(/volume/allocation)", ctxt);
+    allocation = virXPathString(conn, "string(./allocation)", ctxt);
     if (allocation) {
-        unit = virXPathString(conn, "string(/volume/allocation/@unit)", ctxt);
+        unit = virXPathString(conn, "string(./allocation/@unit)", ctxt);
         if (virStorageSize(conn, unit, allocation, &ret->allocation) < 0)
             goto cleanup;
         VIR_FREE(allocation);
@@ -980,9 +998,9 @@ virStorageVolDefParseDoc(virConnectPtr conn,
         ret->allocation = ret->capacity;
     }
 
-    ret->target.path = virXPathString(conn, "string(/volume/target/path)", ctxt);
+    ret->target.path = virXPathString(conn, "string(./target/path)", ctxt);
     if (options->formatFromString) {
-        char *format = virXPathString(conn, "string(/volume/target/format/@type)", ctxt);
+        char *format = virXPathString(conn, "string(./target/format/@type)", ctxt);
         if (format == NULL)
             ret->target.format = options->defaultFormat;
         else
@@ -997,14 +1015,15 @@ virStorageVolDefParseDoc(virConnectPtr conn,
         VIR_FREE(format);
     }
 
-    if (virStorageVolDefParsePerms(conn, ctxt, &ret->target.perms) < 0)
+    if (virStorageDefParsePerms(conn, ctxt, &ret->target.perms,
+                                "./target/permissions", 0600) < 0)
         goto cleanup;
 
 
 
-    ret->backingStore.path = virXPathString(conn, "string(/volume/backingStore/path)", ctxt);
+    ret->backingStore.path = virXPathString(conn, "string(./backingStore/path)", ctxt);
     if (options->formatFromString) {
-        char *format = virXPathString(conn, "string(/volume/backingStore/format/@type)", ctxt);
+        char *format = virXPathString(conn, "string(./backingStore/format/@type)", ctxt);
         if (format == NULL)
             ret->backingStore.format = options->defaultFormat;
         else
@@ -1019,7 +1038,8 @@ virStorageVolDefParseDoc(virConnectPtr conn,
         VIR_FREE(format);
     }
 
-    if (virStorageVolDefParsePerms(conn, ctxt, &ret->backingStore.perms) < 0)
+    if (virStorageDefParsePerms(conn, ctxt, &ret->backingStore.perms,
+                                "./backingStore/permissions", 0600) < 0)
         goto cleanup;
 
     return ret;
@@ -1032,8 +1052,34 @@ virStorageVolDefParseDoc(virConnectPtr conn,
     return NULL;
 }
 
-
 virStorageVolDefPtr
+virStorageVolDefParseNode(virConnectPtr conn,
+                          virStoragePoolDefPtr pool,
+                          xmlDocPtr xml,
+                          xmlNodePtr root) {
+    xmlXPathContextPtr ctxt = NULL;
+    virStorageVolDefPtr def = NULL;
+
+    if (STRNEQ((const char *)root->name, "volume")) {
+        virStorageReportError(conn, VIR_ERR_XML_ERROR,
+                          "%s", _("unknown root element for storage vol"));
+        goto cleanup;
+    }
+
+    ctxt = xmlXPathNewContext(xml);
+    if (ctxt == NULL) {
+        virReportOOMError(conn);
+        goto cleanup;
+    }
+
+    ctxt->node = root;
+    def = virStorageVolDefParseXML(conn, pool, ctxt);
+cleanup:
+    xmlXPathFreeContext(ctxt);
+    return def;
+}
+
+static virStorageVolDefPtr
 virStorageVolDefParse(virConnectPtr conn,
                       virStoragePoolDefPtr pool,
                       const char *xmlStr,
@@ -1042,7 +1088,6 @@ virStorageVolDefParse(virConnectPtr conn,
     xmlParserCtxtPtr pctxt;
     xmlDocPtr xml = NULL;
     xmlNodePtr node = NULL;
-    xmlXPathContextPtr ctxt = NULL;
 
     /* Set up a parser context so we can catch the details of XML errors. */
     pctxt = xmlNewParserCtxt ();
@@ -1052,10 +1097,18 @@ virStorageVolDefParse(virConnectPtr conn,
     pctxt->_private = conn;
 
     if (conn) virResetError (&conn->err);
-    xml = xmlCtxtReadDoc (pctxt, BAD_CAST xmlStr,
-                          filename ? filename : "storage.xml", NULL,
-                          XML_PARSE_NOENT | XML_PARSE_NONET |
-                          XML_PARSE_NOWARNING);
+
+    if (filename) {
+        xml = xmlCtxtReadFile (pctxt, filename, NULL,
+                               XML_PARSE_NOENT | XML_PARSE_NONET |
+                               XML_PARSE_NOWARNING);
+    } else {
+        xml = xmlCtxtReadDoc (pctxt, BAD_CAST xmlStr,
+                              "storage.xml", NULL,
+                              XML_PARSE_NOENT | XML_PARSE_NONET |
+                              XML_PARSE_NOWARNING);
+    }
+
     if (!xml) {
         if (conn && conn->err.code == VIR_ERR_NONE)
               virStorageReportError(conn, VIR_ERR_XML_ERROR,
@@ -1063,12 +1116,6 @@ virStorageVolDefParse(virConnectPtr conn,
         goto cleanup;
     }
 
-    ctxt = xmlXPathNewContext(xml);
-    if (ctxt == NULL) {
-        virReportOOMError(conn);
-        goto cleanup;
-    }
-
     node = xmlDocGetRootElement(xml);
     if (node == NULL) {
         virStorageReportError(conn, VIR_ERR_XML_ERROR,
@@ -1076,21 +1123,34 @@ virStorageVolDefParse(virConnectPtr conn,
         goto cleanup;
     }
 
-    ret = virStorageVolDefParseDoc(conn, pool, ctxt, node);
+    ret = virStorageVolDefParseNode(conn, pool, xml, node);
 
     xmlFreeParserCtxt (pctxt);
-    xmlXPathFreeContext(ctxt);
     xmlFreeDoc(xml);
 
     return ret;
 
  cleanup:
     xmlFreeParserCtxt (pctxt);
-    xmlXPathFreeContext(ctxt);
     xmlFreeDoc(xml);
     return NULL;
 }
 
+virStorageVolDefPtr
+virStorageVolDefParseString(virConnectPtr conn,
+                            virStoragePoolDefPtr pool,
+                            const char *xmlStr)
+{
+    return virStorageVolDefParse(conn, pool, xmlStr, NULL);
+}
+
+virStorageVolDefPtr
+virStorageVolDefParseFile(virConnectPtr conn,
+                          virStoragePoolDefPtr pool,
+                          const char *filename)
+{
+    return virStorageVolDefParse(conn, pool, NULL, filename);
+}
 
 static int
 virStorageVolTargetDefFormat(virConnectPtr conn,
@@ -1332,40 +1392,36 @@ virStoragePoolObjLoad(virConnectPtr conn,
                       virStoragePoolObjListPtr pools,
                       const char *file,
                       const char *path,
-                      const char *xml,
                       const char *autostartLink) {
     virStoragePoolDefPtr def;
     virStoragePoolObjPtr pool;
 
-    if (!(def = virStoragePoolDefParse(NULL, xml, file))) {
-        virErrorPtr err = virGetLastError();
-        virStorageLog("Error parsing storage pool config '%s' : %s",
-                      path, err ? err->message : NULL);
+    if (!(def = virStoragePoolDefParseFile(conn, path))) {
         return NULL;
     }
 
     if (!virFileMatchesNameSuffix(file, def->name, ".xml")) {
-        virStorageLog("Storage pool config filename '%s' does not match pool name '%s'",
+        virStorageError(conn, VIR_ERR_INVALID_STORAGE_POOL,
+            "Storage pool config filename '%s' does not match pool name '%s'",
                       path, def->name);
         virStoragePoolDefFree(def);
         return NULL;
     }
 
     if (!(pool = virStoragePoolObjAssignDef(conn, pools, def))) {
-        virStorageLog("Failed to load storage pool config '%s': out of memory", path);
         virStoragePoolDefFree(def);
         return NULL;
     }
 
     pool->configFile = strdup(path);
     if (pool->configFile == NULL) {
-        virStorageLog("Failed to load storage pool config '%s': out of memory", path);
+        virReportOOMError(conn);
         virStoragePoolDefFree(def);
         return NULL;
     }
     pool->autostartLink = strdup(autostartLink);
     if (pool->autostartLink == NULL) {
-        virStorageLog("Failed to load storage pool config '%s': out of memory", path);
+        virReportOOMError(conn);
         virStoragePoolDefFree(def);
         return NULL;
     }
@@ -1386,16 +1442,14 @@ virStoragePoolLoadAllConfigs(virConnectPtr conn,
     struct dirent *entry;
 
     if (!(dir = opendir(configDir))) {
-        char ebuf[1024];
         if (errno == ENOENT)
             return 0;
-        virStorageLog("Failed to open dir '%s': %s",
-                      configDir, virStrerror(errno, ebuf, sizeof ebuf));
+        virReportSystemError(conn, errno, _("Failed to open dir '%s'"),
+                             configDir);
         return -1;
     }
 
     while ((entry = readdir(dir))) {
-        char *xml = NULL;
         char path[PATH_MAX];
         char autostartLink[PATH_MAX];
         virStoragePoolObjPtr pool;
@@ -1408,26 +1462,24 @@ virStoragePoolLoadAllConfigs(virConnectPtr conn,
 
         if (virFileBuildPath(configDir, entry->d_name,
                              NULL, path, PATH_MAX) < 0) {
-            virStorageLog("Config filename '%s/%s' is too long",
-                          configDir, entry->d_name);
+            virStorageError(conn, VIR_ERR_INTERNAL_ERROR,
+                            "Config filename '%s/%s' is too long",
+                            configDir, entry->d_name);
             continue;
         }
 
         if (virFileBuildPath(autostartDir, entry->d_name,
                              NULL, autostartLink, PATH_MAX) < 0) {
-            virStorageLog("Autostart link path '%s/%s' is too long",
-                          autostartDir, entry->d_name);
+            virStorageError(conn, VIR_ERR_INTERNAL_ERROR,
+                            "Autostart link path '%s/%s' is too long",
+                            autostartDir, entry->d_name);
             continue;
         }
 
-        if (virFileReadAll(path, 8192, &xml) < 0)
-            continue;
-
-        pool = virStoragePoolObjLoad(conn, pools, entry->d_name, path, xml, autostartLink);
+        pool = virStoragePoolObjLoad(conn, pools, entry->d_name, path,
+                                     autostartLink);
         if (pool)
             virStoragePoolObjUnlock(pool);
-
-        VIR_FREE(xml);
     }
 
     closedir(dir);
