@@ -35,12 +35,12 @@
 static int getFreeVethName(char *veth, int maxLen, int startDev)
 {
     int rc = -1;
-    int devNum = startDev;
+    int devNum = startDev-1;
     char path[PATH_MAX];
 
     do {
-        snprintf(path, PATH_MAX, "/sys/class/net/veth%d/", devNum);
         ++devNum;
+        snprintf(path, PATH_MAX, "/sys/class/net/veth%d/", devNum);
     } while (virFileExists(path));
 
     snprintf(veth, maxLen, "veth%d", devNum);
@@ -97,6 +97,7 @@ int vethCreate(char* veth1, int veth1MaxLen,
 
     while ((1 > strlen(veth2)) || STREQ(veth1, veth2)) {
         vethDev = getFreeVethName(veth2, veth2MaxLen, vethDev);
+        ++vethDev;
         DEBUG("assigned veth2: %s", veth2);
     }
 
@@ -190,16 +191,16 @@ error_out:
  *
  * Returns 0 on success or -1 in case of error
  */
-int moveInterfaceToNetNs(const char* interface, int pidInNs)
+int moveInterfaceToNetNs(const char* iface, int pidInNs)
 {
     int rc = -1;
     char *pid = NULL;
     const char *argv[] = {
-        "ip", "link", "set", interface, "netns", NULL, NULL
+        "ip", "link", "set", iface, "netns", NULL, NULL
     };
     int cmdResult;
 
-    if (NULL == interface) {
+    if (NULL == iface) {
         goto error_out;
     }
 
@@ -215,4 +216,3 @@ error_out:
     VIR_FREE(pid);
     return rc;
 }
-
