@@ -19,6 +19,6 @@ int vethCreate(char* veth1, int veth1MaxLen, char* veth2,
                int veth2MaxLen);
 int vethDelete(const char* veth);
 int vethInterfaceUpOrDown(const char* veth, int upOrDown);
-int moveInterfaceToNetNs(const char *interface, int pidInNs);
+int moveInterfaceToNetNs(const char *iface, int pidInNs);
 
 #endif /* VETH_H */
