@@ -124,6 +124,9 @@ static const char *virErrorDomainName(virErrorDomain domain) {
         case VIR_FROM_CONF:
             dom = "Config ";
             break;
+        case VIR_FROM_PHYP:
+            dom = "IBM power hypervisor ";
+            break;
         case VIR_FROM_OPENVZ:
             dom = "OpenVZ ";
             break;
@@ -154,6 +157,18 @@ static const char *virErrorDomainName(virErrorDomain domain) {
         case VIR_FROM_SECURITY:
             dom = "Security Labeling ";
             break;
+        case VIR_FROM_VBOX:
+            dom = "VBOX ";
+            break;
+        case VIR_FROM_INTERFACE:
+            dom = "Interface ";
+            break;
+        case VIR_FROM_ONE:
+            dom = "ONE ";
+            break;
+        case VIR_FROM_ESX:
+            dom = "ESX ";
+            break;
     }
     return(dom);
 }
@@ -174,6 +189,13 @@ virLastErrFreeData(void *data)
 }
 
 
+/**
+ * virErrorInitialize:
+ *
+ * Initialize the error data (per thread)
+ *
+ * Returns 0 in case of success, -1 in case of failure.
+ */
 int
 virErrorInitialize(void)
 {
@@ -531,7 +553,8 @@ virDefaultErrorFunc(virErrorPtr err)
                 dom, lvl, domain, network, err->message);
 }
 
-/*
+/**
+ * virSetGlobalError:
  * Internal helper to ensure the global error object
  * is initialized with a generic message if not already
  * set.
@@ -545,7 +568,10 @@ virSetGlobalError(void)
         virErrorGenericFailure(err);
 }
 
-/*
+/**
+ * virSetConnError:
+ * @conn: pointer to the hypervisor connection
+ *
  * Internal helper to ensure the connection error object
  * is initialized from the global object.
  */
@@ -570,10 +596,11 @@ virSetConnError(virConnectPtr conn)
 
 
 /**
- * virRaiseError:
+ * virRaiseErrorFull:
  * @conn: the connection to the hypervisor if available
- * @dom: the domain if available
- * @net: the network if available
+ * @filename: filename where error was raised
+ * @funcname: function name where error was raised
+ * @linenr: line number where error was raised
  * @domain: the virErrorDomain indicating where it's coming from
  * @code: the virErrorNumber code for the error
  * @level: the virErrorLevel for the error
@@ -582,19 +609,26 @@ virSetConnError(virConnectPtr conn)
  * @str3: extra string info
  * @int1: extra int info
  * @int2: extra int info
- * @msg:  the message to display/transmit
+ * @fmt:  the message to display/transmit
  * @...:  extra parameters for the message display
  *
  * Internal routine called when an error is detected. It will raise it
  * immediately if a callback is found and store it for later handling.
  */
 void
-virRaiseError(virConnectPtr conn,
-              virDomainPtr dom ATTRIBUTE_UNUSED,
-              virNetworkPtr net ATTRIBUTE_UNUSED,
-              int domain, int code, virErrorLevel level,
-              const char *str1, const char *str2, const char *str3,
-              int int1, int int2, const char *msg, ...)
+virRaiseErrorFull(virConnectPtr conn,
+                  const char *filename ATTRIBUTE_UNUSED,
+                  const char *funcname,
+                  size_t linenr,
+                  int domain,
+                  int code,
+                  virErrorLevel level,
+                  const char *str1,
+                  const char *str2,
+                  const char *str3,
+                  int int1,
+                  int int2,
+                  const char *fmt, ...)
 {
     virErrorPtr to;
     void *userData = virUserData;
@@ -630,18 +664,18 @@ virRaiseError(virConnectPtr conn,
     /*
      * formats the message
      */
-    if (msg == NULL) {
+    if (fmt == NULL) {
         str = strdup(_("No error message provided"));
     } else {
-        VIR_GET_VAR_STR(msg, str);
+        VIR_GET_VAR_STR(fmt, str);
     }
 
     /*
      * Hook up the error or warning to the logging facility
-     * TODO: pass function name and lineno
+     * XXXX should we include filename as 'category' instead of domain name ?
      */
     virLogMessage(virErrorDomainName(domain), virErrorLevelPriority(level),
-                  NULL, 0, 1, "%s", str);
+                  funcname, linenr, 1, "%s", str);
 
     /*
      * Save the information about the error
@@ -709,9 +743,9 @@ virErrorMsg(virErrorNumber error, const char *info)
             break;
         case VIR_ERR_NO_CONNECT:
             if (info == NULL)
-                errmsg = _("could not connect to hypervisor");
+                errmsg = _("no hypervisor driver available");
             else
-                errmsg = _("could not connect to %s");
+                errmsg = _("no hypervisor driver available for %s");
             break;
         case VIR_ERR_INVALID_CONN:
             if (info == NULL)
@@ -1004,6 +1038,36 @@ virErrorMsg(virErrorNumber error, const char *info)
             else
                     errmsg = _("Security model not found: %s");
             break;
+        case VIR_ERR_OPERATION_INVALID:
+            if (info == NULL)
+                    errmsg = _("Requested operation is not valid");
+            else
+                    errmsg = _("Requested operation is not valid: %s");
+            break;
+        case VIR_WAR_NO_INTERFACE:
+            if (info == NULL)
+                errmsg = _("Failed to find the interface");
+            else
+                errmsg = _("Failed to find the interface: %s");
+            break;
+        case VIR_ERR_NO_INTERFACE:
+            if (info == NULL)
+                errmsg = _("Interface not found");
+            else
+                errmsg = _("Interface not found: %s");
+            break;
+        case VIR_ERR_INVALID_INTERFACE:
+            if (info == NULL)
+                errmsg = _("invalid interface pointer in");
+            else
+                errmsg = _("invalid interface pointer in %s");
+            break;
+        case VIR_ERR_MULTIPLE_INTERFACES:
+            if (info == NULL)
+                errmsg = _("multiple matching interfaces found");
+            else
+                errmsg = _("multiple matching interfaces found: %s");
+            break;
     }
     return (errmsg);
 }
@@ -1023,10 +1087,12 @@ virErrorMsg(virErrorNumber error, const char *info)
  * Helper function to do most of the grunt work for individual driver
  * ReportError
  */
-void virReportErrorHelper(virConnectPtr conn, int domcode, int errcode,
-                          const char *filename ATTRIBUTE_UNUSED,
-                          const char *funcname ATTRIBUTE_UNUSED,
-                          size_t linenr ATTRIBUTE_UNUSED,
+void virReportErrorHelper(virConnectPtr conn,
+                          int domcode,
+                          int errcode,
+                          const char *filename,
+                          const char *funcname,
+                          size_t linenr,
                           const char *fmt, ...)
 {
     va_list args;
@@ -1042,11 +1108,24 @@ void virReportErrorHelper(virConnectPtr conn, int domcode, int errcode,
     }
 
     virerr = virErrorMsg(errcode, (errorMessage[0] ? errorMessage : NULL));
-    virRaiseError(conn, NULL, NULL, domcode, errcode, VIR_ERR_ERROR,
-                  virerr, errorMessage, NULL, -1, -1, virerr, errorMessage);
+    virRaiseErrorFull(conn, filename, funcname, linenr,
+                      domcode, errcode, VIR_ERR_ERROR,
+                      virerr, errorMessage, NULL,
+                      -1, -1, virerr, errorMessage);
 
 }
 
+/**
+ * virStrerror:
+ * @theerrno: the errno value
+ * @errBuf: the buffer to save the error to
+ * @errBufLen: the buffer length
+ *
+ * Generate an erro string for the given errno
+ *
+ * Returns a pointer to the error string, possibly indicating that the
+ *         error is unknown
+ */
 const char *virStrerror(int theerrno, char *errBuf, size_t errBufLen)
 {
 #ifdef HAVE_STRERROR_R
@@ -1069,12 +1148,25 @@ const char *virStrerror(int theerrno, char *errBuf, size_t errBufLen)
 #endif
 }
 
+/**
+ * virReportSystemErrorFull:
+ * @conn: the hyperisor connection
+ * @domcode: the virErrorDomain indicating where it's coming from
+ * @theerrno: an errno number
+ * @filename: filename where error was raised
+ * @funcname: function name where error was raised
+ * @linenr: line number where error was raised
+ * @fmt:  the message to display/transmit
+ * @...:  extra parameters for the message display
+ *
+ * Convenience internal routine called when a system error is detected.
+ */
 void virReportSystemErrorFull(virConnectPtr conn,
                               int domcode,
                               int theerrno,
-                              const char *filename ATTRIBUTE_UNUSED,
-                              const char *funcname ATTRIBUTE_UNUSED,
-                              size_t linenr ATTRIBUTE_UNUSED,
+                              const char *filename,
+                              const char *funcname,
+                              size_t linenr,
                               const char *fmt, ...)
 {
     char strerror_buf[1024];
@@ -1104,19 +1196,32 @@ void virReportSystemErrorFull(virConnectPtr conn,
     if (!msgDetail)
         msgDetail = errnoDetail;
 
-    virRaiseError(conn, NULL, NULL, domcode, VIR_ERR_SYSTEM_ERROR, VIR_ERR_ERROR,
-                  msg, msgDetail, NULL, -1, -1, msg, msgDetail);
+    virRaiseErrorFull(conn, filename, funcname, linenr,
+                      domcode, VIR_ERR_SYSTEM_ERROR, VIR_ERR_ERROR,
+                      msg, msgDetail, NULL, -1, -1, msg, msgDetail);
 }
 
+/**
+ * virReportOOMErrorFull:
+ * @conn: the hyperisor connection
+ * @domcode: the virErrorDomain indicating where it's coming from
+ * @filename: filename where error was raised
+ * @funcname: function name where error was raised
+ * @linenr: line number where error was raised
+ *
+ * Convenience internal routine called when an out of memory error is
+ * detected
+ */
 void virReportOOMErrorFull(virConnectPtr conn,
                            int domcode,
-                           const char *filename ATTRIBUTE_UNUSED,
-                           const char *funcname ATTRIBUTE_UNUSED,
-                           size_t linenr ATTRIBUTE_UNUSED)
+                           const char *filename,
+                           const char *funcname,
+                           size_t linenr)
 {
     const char *virerr;
 
     virerr = virErrorMsg(VIR_ERR_NO_MEMORY, NULL);
-    virRaiseError(conn, NULL, NULL, domcode, VIR_ERR_NO_MEMORY, VIR_ERR_ERROR,
-                  virerr, NULL, NULL, -1, -1, virerr, NULL);
+    virRaiseErrorFull(conn, filename, funcname, linenr,
+                      domcode, VIR_ERR_NO_MEMORY, VIR_ERR_ERROR,
+                      virerr, NULL, NULL, -1, -1, virerr, NULL);
 }
