@@ -43,6 +43,8 @@
 #include "xml.h"
 #include "util.h"
 #include "memory.h"
+#include "node_device_conf.h"
+#include "pci.h"
 
 #define VIR_FROM_THIS VIR_FROM_XEN
 
@@ -173,7 +175,7 @@ done:
 #ifdef WITH_LIBVIRTD
 
 static int
-xenInitialize (void)
+xenInitialize (int privileged ATTRIBUTE_UNUSED)
 {
     inside_daemon = 1;
     return 0;
@@ -239,25 +241,46 @@ xenUnifiedOpen (virConnectPtr conn, virConnectAuthPtr auth, int flags)
             virReportOOMError (NULL);
             return VIR_DRV_OPEN_ERROR;
         }
+    } else {
+        if (conn->uri->scheme) {
+            /* Decline any scheme which isn't "xen://" or "http://". */
+            if (STRCASENEQ(conn->uri->scheme, "xen") &&
+                STRCASENEQ(conn->uri->scheme, "http"))
+                return VIR_DRV_OPEN_DECLINED;
+
+
+            /* Return an error if the path isn't '' or '/' */
+            if (conn->uri->path &&
+                STRNEQ(conn->uri->path, "") &&
+                STRNEQ(conn->uri->path, "/")) {
+                xenUnifiedError(NULL, VIR_ERR_INTERNAL_ERROR,
+                                _("unexpected Xen URI path '%s', try xen:///"),
+                                conn->uri->path);
+                return VIR_DRV_OPEN_ERROR;
+            }
+
+            /* Decline any xen:// URI with a server specified, allowing remote
+             * driver to handle, but keep any http:/// URIs */
+            if (STRCASEEQ(conn->uri->scheme, "xen") &&
+                conn->uri->server)
+                return VIR_DRV_OPEN_DECLINED;
+        } else {
+            /* Special case URI for Xen driver only:
+             *
+             * Treat a plain path as a Xen UNIX socket path, and give
+             * error unless path is absolute
+             */
+            if (!conn->uri->path || conn->uri->path[0] != '/') {
+                xenUnifiedError(NULL, VIR_ERR_INTERNAL_ERROR,
+                                _("unexpected Xen URI path '%s', try ///var/lib/xen/xend-socket"),
+                                NULLSTR(conn->uri->path));
+                return VIR_DRV_OPEN_ERROR;
+            }
+        }
     }
 
-    /* Refuse any scheme which isn't "xen://" or "http://". */
-    if (conn->uri->scheme &&
-        STRCASENEQ(conn->uri->scheme, "xen") &&
-        STRCASENEQ(conn->uri->scheme, "http"))
-        return VIR_DRV_OPEN_DECLINED;
-
-    /* xmlParseURI will parse a naked string like "foo" as a URI with
-     * a NULL scheme.  That's not useful for us because we want to only
-     * allow full pathnames (eg. ///var/lib/xen/xend-socket).  Decline
-     * anything else.
-     */
-    if (!conn->uri->scheme && (!conn->uri->path || conn->uri->path[0] != '/'))
-        return VIR_DRV_OPEN_DECLINED;
-
-    /* Refuse any xen:// URI with a server specified - allow remote to do it */
-    if (conn->uri->scheme && STRCASEEQ(conn->uri->scheme, "xen") && conn->uri->server)
-        return VIR_DRV_OPEN_DECLINED;
+    /* We now know the URI is definitely for this driver, so beyond
+     * here, don't return DECLINED, always use ERROR */
 
     /* Allocate per-connection private data. */
     if (VIR_ALLOC(priv) < 0) {
@@ -1041,6 +1064,97 @@ xenUnifiedDomainDumpXML (virDomainPtr dom, int flags)
     return NULL;
 }
 
+
+static char *
+xenUnifiedDomainXMLFromNative(virConnectPtr conn,
+                              const char *format,
+                              const char *config,
+                              unsigned int flags ATTRIBUTE_UNUSED)
+{
+    virDomainDefPtr def = NULL;
+    char *ret = NULL;
+    virConfPtr conf = NULL;
+    GET_PRIVATE(conn);
+
+    if (STRNEQ(format, XEN_CONFIG_FORMAT_XM) &&
+        STRNEQ(format, XEN_CONFIG_FORMAT_SEXPR)) {
+        xenUnifiedError(conn, VIR_ERR_INVALID_ARG,
+                        _("unsupported config type %s"), format);
+        return NULL;
+    }
+
+    if (STREQ(format, XEN_CONFIG_FORMAT_XM)) {
+        conf = virConfReadMem(config, strlen(config), 0);
+        if (!conf)
+            goto cleanup;
+
+        def = xenXMDomainConfigParse(conn, conf);
+    } else if (STREQ(format, XEN_CONFIG_FORMAT_SEXPR)) {
+        def = xenDaemonParseSxprString(conn, config, priv->xendConfigVersion);
+    }
+    if (!def)
+        goto cleanup;
+
+    ret = virDomainDefFormat(conn, def, 0);
+
+cleanup:
+    virDomainDefFree(def);
+    return ret;
+}
+
+
+#define MAX_CONFIG_SIZE (1024 * 65)
+static char *
+xenUnifiedDomainXMLToNative(virConnectPtr conn,
+                            const char *format,
+                            const char *xmlData,
+                            unsigned int flags ATTRIBUTE_UNUSED)
+{
+    virDomainDefPtr def = NULL;
+    char *ret = NULL;
+    virConfPtr conf = NULL;
+    GET_PRIVATE(conn);
+
+    if (STRNEQ(format, XEN_CONFIG_FORMAT_XM) &&
+        STRNEQ(format, XEN_CONFIG_FORMAT_SEXPR)) {
+        xenUnifiedError(conn, VIR_ERR_INVALID_ARG,
+                        _("unsupported config type %s"), format);
+        goto cleanup;
+    }
+
+    if (!(def = virDomainDefParseString(conn,
+                                        priv->caps,
+                                        xmlData,
+                                        0)))
+        goto cleanup;
+
+    if (STREQ(format, XEN_CONFIG_FORMAT_XM)) {
+        int len = MAX_CONFIG_SIZE;
+        conf = xenXMDomainConfigFormat(conn, def);
+        if (!conf)
+            goto cleanup;
+
+        if (VIR_ALLOC_N(ret, len) < 0) {
+            virReportOOMError(conn);
+            goto cleanup;
+        }
+
+        if (virConfWriteMem(ret, &len, conf) < 0) {
+            VIR_FREE(ret);
+            goto cleanup;
+        }
+    } else if (STREQ(format, XEN_CONFIG_FORMAT_SEXPR)) {
+        ret = xenDaemonFormatSxpr(conn, def, priv->xendConfigVersion);
+    }
+
+cleanup:
+    virDomainDefFree(def);
+    if (conf)
+        virConfFree(conf);
+    return ret;
+}
+
+
 static int
 xenUnifiedDomainMigratePrepare (virConnectPtr dconn,
                                 char **cookie,
@@ -1420,67 +1534,194 @@ xenUnifiedDomainEventDeregister (virConnectPtr conn,
     return ret;
 }
 
+
+static int
+xenUnifiedNodeDeviceGetPciInfo (virNodeDevicePtr dev,
+                           unsigned *domain,
+                           unsigned *bus,
+                           unsigned *slot,
+                           unsigned *function)
+{
+    virNodeDeviceDefPtr def = NULL;
+    virNodeDevCapsDefPtr cap;
+    char *xml = NULL;
+    int ret = -1;
+
+    xml = virNodeDeviceGetXMLDesc(dev, 0);
+    if (!xml)
+        goto out;
+
+    def = virNodeDeviceDefParseString(dev->conn, xml, EXISTING_DEVICE);
+    if (!def)
+        goto out;
+
+    cap = def->caps;
+    while (cap) {
+        if (cap->type == VIR_NODE_DEV_CAP_PCI_DEV) {
+            *domain   = cap->data.pci_dev.domain;
+            *bus      = cap->data.pci_dev.bus;
+            *slot     = cap->data.pci_dev.slot;
+            *function = cap->data.pci_dev.function;
+            break;
+        }
+
+        cap = cap->next;
+    }
+
+    if (!cap) {
+        xenUnifiedError(dev->conn, VIR_ERR_INVALID_ARG,
+                        _("device %s is not a PCI device"), dev->name);
+        goto out;
+    }
+
+    ret = 0;
+out:
+    virNodeDeviceDefFree(def);
+    VIR_FREE(xml);
+    return ret;
+}
+
+static int
+xenUnifiedNodeDeviceDettach (virNodeDevicePtr dev)
+{
+    pciDevice *pci;
+    unsigned domain, bus, slot, function;
+    int ret = -1;
+
+    if (xenUnifiedNodeDeviceGetPciInfo(dev, &domain, &bus, &slot, &function) < 0)
+        return -1;
+
+    pci = pciGetDevice(dev->conn, domain, bus, slot, function);
+    if (!pci)
+        return -1;
+
+    if (pciDettachDevice(dev->conn, pci) < 0)
+        goto out;
+
+    ret = 0;
+out:
+    pciFreeDevice(dev->conn, pci);
+    return ret;
+}
+
+static int
+xenUnifiedNodeDeviceReAttach (virNodeDevicePtr dev)
+{
+    pciDevice *pci;
+    unsigned domain, bus, slot, function;
+    int ret = -1;
+
+    if (xenUnifiedNodeDeviceGetPciInfo(dev, &domain, &bus, &slot, &function) < 0)
+        return -1;
+
+    pci = pciGetDevice(dev->conn, domain, bus, slot, function);
+    if (!pci)
+        return -1;
+
+    if (pciReAttachDevice(dev->conn, pci) < 0)
+        goto out;
+
+    ret = 0;
+out:
+    pciFreeDevice(dev->conn, pci);
+    return ret;
+}
+
+static int
+xenUnifiedNodeDeviceReset (virNodeDevicePtr dev)
+{
+    pciDevice *pci;
+    unsigned domain, bus, slot, function;
+    int ret = -1;
+
+    if (xenUnifiedNodeDeviceGetPciInfo(dev, &domain, &bus, &slot, &function) < 0)
+        return -1;
+
+    pci = pciGetDevice(dev->conn, domain, bus, slot, function);
+    if (!pci)
+        return -1;
+
+    if (pciResetDevice(dev->conn, pci) < 0)
+        goto out;
+
+    ret = 0;
+out:
+    pciFreeDevice(dev->conn, pci);
+    return ret;
+}
+
+
 /*----- Register with libvirt.c, and initialise Xen drivers. -----*/
 
 /* The interface which we export upwards to libvirt.c. */
 static virDriver xenUnifiedDriver = {
-    .no = VIR_DRV_XEN_UNIFIED,
-    .name = "Xen",
-    .open 			= xenUnifiedOpen,
-    .close 			= xenUnifiedClose,
-    .supports_feature   = xenUnifiedSupportsFeature,
-    .type 			= xenUnifiedType,
-    .version 			= xenUnifiedGetVersion,
-    .getHostname    = xenUnifiedGetHostname,
-    .getMaxVcpus 			= xenUnifiedGetMaxVcpus,
-    .nodeGetInfo 			= xenUnifiedNodeGetInfo,
-    .getCapabilities 		= xenUnifiedGetCapabilities,
-    .listDomains 			= xenUnifiedListDomains,
-    .numOfDomains 		= xenUnifiedNumOfDomains,
-    .domainCreateXML 		= xenUnifiedDomainCreateXML,
-    .domainLookupByID 		= xenUnifiedDomainLookupByID,
-    .domainLookupByUUID 		= xenUnifiedDomainLookupByUUID,
-    .domainLookupByName 		= xenUnifiedDomainLookupByName,
-    .domainSuspend 		= xenUnifiedDomainSuspend,
-    .domainResume 		= xenUnifiedDomainResume,
-    .domainShutdown 		= xenUnifiedDomainShutdown,
-    .domainReboot 		= xenUnifiedDomainReboot,
-    .domainDestroy 		= xenUnifiedDomainDestroy,
-    .domainGetOSType 		= xenUnifiedDomainGetOSType,
-    .domainGetMaxMemory 		= xenUnifiedDomainGetMaxMemory,
-    .domainSetMaxMemory 		= xenUnifiedDomainSetMaxMemory,
-    .domainSetMemory 		= xenUnifiedDomainSetMemory,
-    .domainGetInfo 		= xenUnifiedDomainGetInfo,
-    .domainSave 			= xenUnifiedDomainSave,
-    .domainRestore 		= xenUnifiedDomainRestore,
-    .domainCoreDump 		= xenUnifiedDomainCoreDump,
-    .domainSetVcpus 		= xenUnifiedDomainSetVcpus,
-    .domainPinVcpu 		= xenUnifiedDomainPinVcpu,
-    .domainGetVcpus 		= xenUnifiedDomainGetVcpus,
-    .domainGetMaxVcpus 		= xenUnifiedDomainGetMaxVcpus,
-    .domainDumpXML 		= xenUnifiedDomainDumpXML,
-    .listDefinedDomains 		= xenUnifiedListDefinedDomains,
-    .numOfDefinedDomains 		= xenUnifiedNumOfDefinedDomains,
-    .domainCreate 		= xenUnifiedDomainCreate,
-    .domainDefineXML 		= xenUnifiedDomainDefineXML,
-    .domainUndefine 		= xenUnifiedDomainUndefine,
-    .domainAttachDevice 		= xenUnifiedDomainAttachDevice,
-    .domainDetachDevice 		= xenUnifiedDomainDetachDevice,
-    .domainGetAutostart             = xenUnifiedDomainGetAutostart,
-    .domainSetAutostart             = xenUnifiedDomainSetAutostart,
-    .domainGetSchedulerType	= xenUnifiedDomainGetSchedulerType,
-    .domainGetSchedulerParameters	= xenUnifiedDomainGetSchedulerParameters,
-    .domainSetSchedulerParameters	= xenUnifiedDomainSetSchedulerParameters,
-    .domainMigratePrepare		= xenUnifiedDomainMigratePrepare,
-    .domainMigratePerform		= xenUnifiedDomainMigratePerform,
-    .domainMigrateFinish		= xenUnifiedDomainMigrateFinish,
-    .domainBlockStats	= xenUnifiedDomainBlockStats,
-    .domainInterfaceStats = xenUnifiedDomainInterfaceStats,
-    .domainBlockPeek	= xenUnifiedDomainBlockPeek,
-    .nodeGetCellsFreeMemory = xenUnifiedNodeGetCellsFreeMemory,
-    .getFreeMemory = xenUnifiedNodeGetFreeMemory,
-    .domainEventRegister = xenUnifiedDomainEventRegister,
-    .domainEventDeregister = xenUnifiedDomainEventDeregister,
+    VIR_DRV_XEN_UNIFIED,
+    "Xen",
+    xenUnifiedOpen, /* open */
+    xenUnifiedClose, /* close */
+    xenUnifiedSupportsFeature, /* supports_feature */
+    xenUnifiedType, /* type */
+    xenUnifiedGetVersion, /* version */
+    xenUnifiedGetHostname, /* getHostname */
+    xenUnifiedGetMaxVcpus, /* getMaxVcpus */
+    xenUnifiedNodeGetInfo, /* nodeGetInfo */
+    xenUnifiedGetCapabilities, /* getCapabilities */
+    xenUnifiedListDomains, /* listDomains */
+    xenUnifiedNumOfDomains, /* numOfDomains */
+    xenUnifiedDomainCreateXML, /* domainCreateXML */
+    xenUnifiedDomainLookupByID, /* domainLookupByID */
+    xenUnifiedDomainLookupByUUID, /* domainLookupByUUID */
+    xenUnifiedDomainLookupByName, /* domainLookupByName */
+    xenUnifiedDomainSuspend, /* domainSuspend */
+    xenUnifiedDomainResume, /* domainResume */
+    xenUnifiedDomainShutdown, /* domainShutdown */
+    xenUnifiedDomainReboot, /* domainReboot */
+    xenUnifiedDomainDestroy, /* domainDestroy */
+    xenUnifiedDomainGetOSType, /* domainGetOSType */
+    xenUnifiedDomainGetMaxMemory, /* domainGetMaxMemory */
+    xenUnifiedDomainSetMaxMemory, /* domainSetMaxMemory */
+    xenUnifiedDomainSetMemory, /* domainSetMemory */
+    xenUnifiedDomainGetInfo, /* domainGetInfo */
+    xenUnifiedDomainSave, /* domainSave */
+    xenUnifiedDomainRestore, /* domainRestore */
+    xenUnifiedDomainCoreDump, /* domainCoreDump */
+    xenUnifiedDomainSetVcpus, /* domainSetVcpus */
+    xenUnifiedDomainPinVcpu, /* domainPinVcpu */
+    xenUnifiedDomainGetVcpus, /* domainGetVcpus */
+    xenUnifiedDomainGetMaxVcpus, /* domainGetMaxVcpus */
+    NULL, /* domainGetSecurityLabel */
+    NULL, /* nodeGetSecurityModel */
+    xenUnifiedDomainDumpXML, /* domainDumpXML */
+    xenUnifiedDomainXMLFromNative, /* domainXmlFromNative */
+    xenUnifiedDomainXMLToNative, /* domainXmlToNative */
+    xenUnifiedListDefinedDomains, /* listDefinedDomains */
+    xenUnifiedNumOfDefinedDomains, /* numOfDefinedDomains */
+    xenUnifiedDomainCreate, /* domainCreate */
+    xenUnifiedDomainDefineXML, /* domainDefineXML */
+    xenUnifiedDomainUndefine, /* domainUndefine */
+    xenUnifiedDomainAttachDevice, /* domainAttachDevice */
+    xenUnifiedDomainDetachDevice, /* domainDetachDevice */
+    xenUnifiedDomainGetAutostart, /* domainGetAutostart */
+    xenUnifiedDomainSetAutostart, /* domainSetAutostart */
+    xenUnifiedDomainGetSchedulerType, /* domainGetSchedulerType */
+    xenUnifiedDomainGetSchedulerParameters, /* domainGetSchedulerParameters */
+    xenUnifiedDomainSetSchedulerParameters, /* domainSetSchedulerParameters */
+    xenUnifiedDomainMigratePrepare, /* domainMigratePrepare */
+    xenUnifiedDomainMigratePerform, /* domainMigratePerform */
+    xenUnifiedDomainMigrateFinish, /* domainMigrateFinish */
+    xenUnifiedDomainBlockStats, /* domainBlockStats */
+    xenUnifiedDomainInterfaceStats, /* domainInterfaceStats */
+    xenUnifiedDomainBlockPeek, /* domainBlockPeek */
+    NULL, /* domainMemoryPeek */
+    xenUnifiedNodeGetCellsFreeMemory, /* nodeGetCellsFreeMemory */
+    xenUnifiedNodeGetFreeMemory, /* getFreeMemory */
+    xenUnifiedDomainEventRegister, /* domainEventRegister */
+    xenUnifiedDomainEventDeregister, /* domainEventDeregister */
+    NULL, /* domainMigratePrepare2 */
+    NULL, /* domainMigrateFinish2 */
+    xenUnifiedNodeDeviceDettach, /* nodeDeviceDettach */
+    xenUnifiedNodeDeviceReAttach, /* nodeDeviceReAttach */
+    xenUnifiedNodeDeviceReset, /* nodeDeviceReset */
 };
 
 /**
