@@ -82,7 +82,6 @@ struct xenUnifiedDriver xenXMDriver = {
     xenXMClose, /* close */
     NULL, /* version */
     NULL, /* hostname */
-    NULL, /* URI */
     NULL, /* nodeGetInfo */
     NULL, /* getCapabilities */
     NULL, /* listDomains */
@@ -123,6 +122,19 @@ struct xenUnifiedDriver xenXMDriver = {
         virReportErrorHelper(conn, VIR_FROM_XENXM, code, __FILE__,         \
                                __FUNCTION__, __LINE__, fmt)
 
+#ifndef WITH_XEN_INOTIFY
+static int xenInotifyActive(virConnectPtr conn ATTRIBUTE_UNUSED)
+{
+   return 0;
+}
+#else
+static int xenInotifyActive(virConnectPtr conn)
+{
+   xenUnifiedPrivatePtr priv = (xenUnifiedPrivatePtr) conn->privateData;
+   return priv->inotifyWatch > 0;
+}
+#endif
+
 /* Convenience method to grab a int from the config file object */
 static int xenXMConfigGetBool(virConnectPtr conn,
                               virConfPtr conf,
@@ -325,7 +337,7 @@ xenXMConfigReadFile(virConnectPtr conn, const char *filename) {
     virConfPtr conf;
     virDomainDefPtr def;
 
-    if (!(conf = virConfReadFile(filename)))
+    if (!(conf = virConfReadFile(filename, 0)))
         return NULL;
 
     def = xenXMDomainConfigParse(conn, conf);
@@ -673,6 +685,7 @@ xenXMDomainConfigParse(virConnectPtr conn, virConfPtr conf) {
     virDomainDiskDefPtr disk = NULL;
     virDomainNetDefPtr net = NULL;
     virDomainGraphicsDefPtr graphics = NULL;
+    virDomainHostdevDefPtr hostdev = NULL;
     int i;
     const char *defaultArch, *defaultMachine;
 
@@ -695,7 +708,7 @@ xenXMDomainConfigParse(virConnectPtr conn, virConfPtr conf) {
     if (!(def->os.type = strdup(hvm ? "hvm" : "xen")))
         goto no_memory;
 
-    defaultArch = virCapabilitiesDefaultGuestArch(priv->caps, def->os.type);
+    defaultArch = virCapabilitiesDefaultGuestArch(priv->caps, def->os.type, virDomainVirtTypeToString(def->virtType));
     if (defaultArch == NULL) {
         xenXMError(conn, VIR_ERR_INTERNAL_ERROR,
                    _("no supported architecture for os type '%s'"),
@@ -818,10 +831,10 @@ xenXMDomainConfigParse(virConnectPtr conn, virConfPtr conf) {
             goto cleanup;
         else if (val)
             def->features |= (1 << VIR_DOMAIN_FEATURE_APIC);
-
-        if (xenXMConfigGetBool(conn, conf, "localtime", &def->localtime, 0) < 0)
-            goto cleanup;
     }
+    if (xenXMConfigGetBool(conn, conf, "localtime", &def->localtime, 0) < 0)
+        goto cleanup;
+
     if (xenXMConfigCopyStringOpt(conn, conf, "device_model", &def->emulator) < 0)
         goto cleanup;
 
@@ -990,6 +1003,7 @@ xenXMDomainConfigParse(virConnectPtr conn, virConfPtr conf) {
             char ip[16];
             char mac[18];
             char bridge[50];
+            char vifname[50];
             char *key;
 
             bridge[0] = '\0';
@@ -997,6 +1011,7 @@ xenXMDomainConfigParse(virConnectPtr conn, virConfPtr conf) {
             script[0] = '\0';
             ip[0] = '\0';
             model[0] = '\0';
+            vifname[0] = '\0';
 
             if ((list->type != VIR_CONF_STRING) || (list->str == NULL))
                 goto skipnic;
@@ -1035,6 +1050,12 @@ xenXMDomainConfigParse(virConnectPtr conn, virConfPtr conf) {
                         len = sizeof(model)-1;
                     strncpy(model, data, len);
                     model[len] = '\0';
+                } else if (STRPREFIX(key, "vifname=")) {
+                    int len = nextkey ? (nextkey - data) : sizeof(vifname)-1;
+                    if (len > (sizeof(vifname)-1))
+                        len = sizeof(vifname)-1;
+                    strncpy(vifname, data, len);
+                    vifname[len] = '\0';
                 } else if (STRPREFIX(key, "ip=")) {
                     int len = nextkey ? (nextkey - data) : 15;
                     if (len > 15)
@@ -1104,6 +1125,10 @@ xenXMDomainConfigParse(virConnectPtr conn, virConfPtr conf) {
                 !(net->model = strdup(model)))
                 goto no_memory;
 
+            if (vifname[0] &&
+                !(net->ifname = strdup(vifname)))
+                goto no_memory;
+
             if (VIR_REALLOC_N(def->nets, def->nnets+1) < 0)
                 goto no_memory;
             def->nets[def->nnets++] = net;
@@ -1115,6 +1140,88 @@ xenXMDomainConfigParse(virConnectPtr conn, virConfPtr conf) {
         }
     }
 
+    list = virConfGetValue(conf, "pci");
+    if (list && list->type == VIR_CONF_LIST) {
+        list = list->list;
+        while (list) {
+            char domain[5];
+            char bus[3];
+            char slot[3];
+            char func[2];
+            char *key, *nextkey;
+            int domainID;
+            int busID;
+            int slotID;
+            int funcID;
+
+            domain[0] = bus[0] = slot[0] = func[0] = '\0';
+
+            if ((list->type != VIR_CONF_STRING) || (list->str == NULL))
+                goto skippci;
+
+            /* pci=['0000:00:1b.0','0000:00:13.0'] */
+            key = list->str;
+            if (!(key = list->str))
+                goto skippci;
+            if (!(nextkey = strchr(key, ':')))
+                goto skippci;
+
+            if ((nextkey - key) > (sizeof(domain)-1))
+                goto skippci;
+
+            strncpy(domain, key, sizeof(domain));
+            domain[sizeof(domain)-1] = '\0';
+
+            key = nextkey + 1;
+            if (!(nextkey = strchr(key, ':')))
+                goto skippci;
+
+            strncpy(bus, key, sizeof(bus));
+            bus[sizeof(bus)-1] = '\0';
+
+            key = nextkey + 1;
+            if (!(nextkey = strchr(key, '.')))
+                goto skippci;
+
+            strncpy(slot, key, sizeof(slot));
+            slot[sizeof(slot)-1] = '\0';
+
+            key = nextkey + 1;
+            if (strlen(key) != 1)
+                goto skippci;
+
+            strncpy(func, key, sizeof(func));
+            func[sizeof(func)-1] = '\0';
+
+            if (virStrToLong_i(domain, NULL, 16, &domainID) < 0)
+                goto skippci;
+            if (virStrToLong_i(bus, NULL, 16, &busID) < 0)
+                goto skippci;
+            if (virStrToLong_i(slot, NULL, 16, &slotID) < 0)
+                goto skippci;
+            if (virStrToLong_i(func, NULL, 16, &funcID) < 0)
+                goto skippci;
+
+            if (VIR_ALLOC(hostdev) < 0)
+                goto cleanup;
+
+            hostdev->managed = 0;
+            hostdev->source.subsys.type = VIR_DOMAIN_HOSTDEV_SUBSYS_TYPE_PCI;
+            hostdev->source.subsys.u.pci.domain = domainID;
+            hostdev->source.subsys.u.pci.bus = busID;
+            hostdev->source.subsys.u.pci.slot = slotID;
+            hostdev->source.subsys.u.pci.function = funcID;
+
+            if (VIR_REALLOC_N(def->hostdevs, def->nhostdevs+1) < 0)
+                goto no_memory;
+            def->hostdevs[def->nhostdevs++] = hostdev;
+            hostdev = NULL;
+
+        skippci:
+            list = list->next;
+        }
+    }
+
     if (hvm) {
         if (xenXMConfigGetString(conn, conf, "usbdevice", &str, NULL) < 0)
             goto cleanup;
@@ -1163,7 +1270,10 @@ xenXMDomainConfigParse(virConnectPtr conn, virConfPtr conf) {
             if (xenXMConfigCopyStringOpt(conn, conf, "keymap", &graphics->data.vnc.keymap) < 0)
                 goto cleanup;
 
-            def->graphics = graphics;
+            if (VIR_ALLOC_N(def->graphics, 1) < 0)
+                goto no_memory;
+            def->graphics[0] = graphics;
+            def->ngraphics = 1;
             graphics = NULL;
         } else {
             if (xenXMConfigGetBool(conn, conf, "sdl", &val, 0) < 0)
@@ -1176,7 +1286,10 @@ xenXMDomainConfigParse(virConnectPtr conn, virConfPtr conf) {
                     goto cleanup;
                 if (xenXMConfigCopyStringOpt(conn, conf, "xauthority", &graphics->data.sdl.xauth) < 0)
                     goto cleanup;
-                def->graphics = graphics;
+                if (VIR_ALLOC_N(def->graphics, 1) < 0)
+                    goto no_memory;
+                def->graphics[0] = graphics;
+                def->ngraphics = 1;
                 graphics = NULL;
             }
         }
@@ -1245,7 +1358,10 @@ xenXMDomainConfigParse(virConnectPtr conn, virConfPtr conf) {
                     nextkey++;
                 key = nextkey;
             }
-            def->graphics = graphics;
+            if (VIR_ALLOC_N(def->graphics, 1) < 0)
+                goto no_memory;
+            def->graphics[0] = graphics;
+            def->ngraphics = 1;
             graphics = NULL;
         }
     }
@@ -1530,7 +1646,7 @@ int xenXMDomainPinVcpu(virDomainPtr domain,
     const char *filename;
     xenXMConfCachePtr entry;
     virBuffer mapbuf = VIR_BUFFER_INITIALIZER;
-    char *mapstr = NULL;
+    char *mapstr = NULL, *mapsave = NULL;
     int i, j, n, comma = 0;
     int ret = -1;
     char *cpuset = NULL;
@@ -1585,6 +1701,7 @@ int xenXMDomainPinVcpu(virDomainPtr domain,
     }
 
     mapstr = virBufferContentAndReset(&mapbuf);
+    mapsave = mapstr;
 
     if (VIR_ALLOC_N(cpuset, maxcpu) < 0) {
         virReportOOMError(domain->conn);
@@ -1606,7 +1723,7 @@ int xenXMDomainPinVcpu(virDomainPtr domain,
     ret = 0;
 
  cleanup:
-    VIR_FREE(mapstr);
+    VIR_FREE(mapsave);
     VIR_FREE(cpuset);
     xenUnifiedUnlock(priv);
     return (ret);
@@ -1633,10 +1750,8 @@ virDomainPtr xenXMDomainLookupByName(virConnectPtr conn, const char *domname) {
     priv = conn->privateData;
     xenUnifiedLock(priv);
 
-#ifndef WITH_XEN_INOTIFY
-    if (xenXMConfigCacheRefresh (conn) < 0)
+    if (!xenInotifyActive(conn) && xenXMConfigCacheRefresh (conn) < 0)
         goto cleanup;
-#endif
 
     if (!(filename = virHashLookup(priv->nameConfigMap, domname)))
         goto cleanup;
@@ -1691,10 +1806,8 @@ virDomainPtr xenXMDomainLookupByUUID(virConnectPtr conn,
     priv = conn->privateData;
     xenUnifiedLock(priv);
 
-#ifndef WITH_XEN_INOTIFY
-    if (xenXMConfigCacheRefresh (conn) < 0)
+    if (!xenInotifyActive(conn) && xenXMConfigCacheRefresh (conn) < 0)
         goto cleanup;
-#endif
 
     if (!(entry = virHashSearch(priv->configCache, xenXMDomainSearchForUUID, (const void *)uuid)))
         goto cleanup;
@@ -1735,11 +1848,8 @@ int xenXMDomainCreate(virDomainPtr domain) {
     if (!(entry = virHashLookup(priv->configCache, filename)))
         goto error;
 
-    if (!(sexpr = xenDaemonFormatSxpr(domain->conn, entry->def, priv->xendConfigVersion))) {
-        xenXMError(domain->conn, VIR_ERR_XML_ERROR,
-                   "%s", _("failed to build sexpr"));
+    if (!(sexpr = xenDaemonFormatSxpr(domain->conn, entry->def, priv->xendConfigVersion)))
         goto error;
-    }
 
     ret = xenDaemonDomainCreateXML(domain->conn, sexpr);
     VIR_FREE(sexpr);
@@ -1888,6 +1998,7 @@ static int xenXMDomainConfigFormatNet(virConnectPtr conn,
         virBufferVSprintf(&buf, ",bridge=%s", net->data.bridge.brname);
         if (net->data.bridge.ipaddr)
             virBufferVSprintf(&buf, ",ip=%s", net->data.bridge.ipaddr);
+        virBufferVSprintf(&buf, ",script=%s", DEFAULT_VIF_SCRIPT);
         break;
 
     case VIR_DOMAIN_NET_TYPE_ETHERNET:
@@ -1898,7 +2009,27 @@ static int xenXMDomainConfigFormatNet(virConnectPtr conn,
         break;
 
     case VIR_DOMAIN_NET_TYPE_NETWORK:
-        break;
+    {
+        virNetworkPtr network = virNetworkLookupByName(conn, net->data.network.name);
+        char *bridge;
+        if (!network) {
+            xenXMError(conn, VIR_ERR_NO_NETWORK, "%s",
+                       net->data.network.name);
+            return -1;
+        }
+        bridge = virNetworkGetBridgeName(network);
+        virNetworkFree(network);
+        if (!bridge) {
+            xenXMError(conn, VIR_ERR_INTERNAL_ERROR,
+                       _("network %s is not active"),
+                       net->data.network.name);
+            return -1;
+        }
+
+        virBufferVSprintf(&buf, ",bridge=%s", bridge);
+        virBufferVSprintf(&buf, ",script=%s", DEFAULT_VIF_SCRIPT);
+    }
+    break;
 
     default:
         xenXMError(conn, VIR_ERR_INTERNAL_ERROR,
@@ -1914,6 +2045,10 @@ static int xenXMDomainConfigFormatNet(virConnectPtr conn,
         virBufferVSprintf(&buf, ",model=%s",
                           net->model);
 
+    if (net->ifname)
+        virBufferVSprintf(&buf, ",vifname=%s",
+                          net->ifname);
+
     if (VIR_ALLOC(val) < 0) {
         virReportOOMError(conn);
         goto cleanup;
@@ -1939,6 +2074,76 @@ cleanup:
 
 
 
+static int
+xenXMDomainConfigFormatPCI(virConnectPtr conn,
+                           virConfPtr conf,
+                           virDomainDefPtr def)
+{
+
+    virConfValuePtr pciVal = NULL;
+    int hasPCI = 0;
+    int i;
+
+    for (i = 0 ; i < def->nhostdevs ; i++)
+        if (def->hostdevs[i]->mode == VIR_DOMAIN_HOSTDEV_MODE_SUBSYS &&
+            def->hostdevs[i]->source.subsys.type == VIR_DOMAIN_HOSTDEV_SUBSYS_TYPE_PCI)
+            hasPCI = 1;
+
+    if (!hasPCI)
+        return 0;
+
+    if (VIR_ALLOC(pciVal) < 0)
+        return -1;
+
+    pciVal->type = VIR_CONF_LIST;
+    pciVal->list = NULL;
+
+    for (i = 0 ; i < def->nhostdevs ; i++) {
+        if (def->hostdevs[i]->mode == VIR_DOMAIN_HOSTDEV_MODE_SUBSYS &&
+            def->hostdevs[i]->source.subsys.type == VIR_DOMAIN_HOSTDEV_SUBSYS_TYPE_PCI) {
+            virConfValuePtr val, tmp;
+            char *buf;
+
+            if (virAsprintf(&buf, "%04x:%02x:%02x.%x",
+                            def->hostdevs[i]->source.subsys.u.pci.domain,
+                            def->hostdevs[i]->source.subsys.u.pci.bus,
+                            def->hostdevs[i]->source.subsys.u.pci.slot,
+                            def->hostdevs[i]->source.subsys.u.pci.function) < 0)
+                goto error;
+
+            if (VIR_ALLOC(val) < 0) {
+                VIR_FREE(buf);
+                virReportOOMError(conn);
+                goto error;
+            }
+            val->type = VIR_CONF_STRING;
+            val->str = buf;
+            tmp = pciVal->list;
+            while (tmp && tmp->next)
+                tmp = tmp->next;
+            if (tmp)
+                tmp->next = val;
+            else
+                pciVal->list = val;
+        }
+    }
+
+    if (pciVal->list != NULL) {
+        int ret = virConfSetValue(conf, "pci", pciVal);
+        pciVal = NULL;
+        if (ret < 0)
+            return -1;
+    }
+    VIR_FREE(pciVal);
+
+    return 0;
+
+error:
+    virConfFreeValue(pciVal);
+    return -1;
+}
+
+
 virConfPtr xenXMDomainConfigFormat(virConnectPtr conn,
                                    virDomainDefPtr def) {
     virConfPtr conf = NULL;
@@ -2118,20 +2323,20 @@ virConfPtr xenXMDomainConfigFormat(virConnectPtr conn,
         }
     }
 
-    if (def->graphics) {
+    if (def->ngraphics == 1) {
         if (priv->xendConfigVersion < (hvm ? 4 : XEND_CONFIG_MIN_VERS_PVFB_NEWCONF)) {
-            if (def->graphics->type == VIR_DOMAIN_GRAPHICS_TYPE_SDL) {
+            if (def->graphics[0]->type == VIR_DOMAIN_GRAPHICS_TYPE_SDL) {
                 if (xenXMConfigSetInt(conf, "sdl", 1) < 0)
                     goto no_memory;
                 if (xenXMConfigSetInt(conf, "vnc", 0) < 0)
                     goto no_memory;
-                if (def->graphics->data.sdl.display &&
+                if (def->graphics[0]->data.sdl.display &&
                     xenXMConfigSetString(conf, "display",
-                                         def->graphics->data.sdl.display) < 0)
+                                     def->graphics[0]->data.sdl.display) < 0)
                     goto no_memory;
-                if (def->graphics->data.sdl.xauth &&
+                if (def->graphics[0]->data.sdl.xauth &&
                     xenXMConfigSetString(conf, "xauthority",
-                                         def->graphics->data.sdl.xauth) < 0)
+                                         def->graphics[0]->data.sdl.xauth) < 0)
                     goto no_memory;
             } else {
                 if (xenXMConfigSetInt(conf, "sdl", 0) < 0)
@@ -2139,53 +2344,53 @@ virConfPtr xenXMDomainConfigFormat(virConnectPtr conn,
                 if (xenXMConfigSetInt(conf, "vnc", 1) < 0)
                     goto no_memory;
                 if (xenXMConfigSetInt(conf, "vncunused",
-                                      def->graphics->data.vnc.autoport ? 1 : 0) < 0)
+                              def->graphics[0]->data.vnc.autoport ? 1 : 0) < 0)
                     goto no_memory;
-                if (!def->graphics->data.vnc.autoport &&
+                if (!def->graphics[0]->data.vnc.autoport &&
                     xenXMConfigSetInt(conf, "vncdisplay",
-                                      def->graphics->data.vnc.port - 5900) < 0)
+                                  def->graphics[0]->data.vnc.port - 5900) < 0)
                     goto no_memory;
-                if (def->graphics->data.vnc.listenAddr &&
+                if (def->graphics[0]->data.vnc.listenAddr &&
                     xenXMConfigSetString(conf, "vnclisten",
-                                         def->graphics->data.vnc.listenAddr) < 0)
+                                    def->graphics[0]->data.vnc.listenAddr) < 0)
                     goto no_memory;
-                if (def->graphics->data.vnc.passwd &&
+                if (def->graphics[0]->data.vnc.passwd &&
                     xenXMConfigSetString(conf, "vncpasswd",
-                                         def->graphics->data.vnc.passwd) < 0)
+                                        def->graphics[0]->data.vnc.passwd) < 0)
                     goto no_memory;
-                if (def->graphics->data.vnc.keymap &&
+                if (def->graphics[0]->data.vnc.keymap &&
                     xenXMConfigSetString(conf, "keymap",
-                                         def->graphics->data.vnc.keymap) < 0)
+                                        def->graphics[0]->data.vnc.keymap) < 0)
                     goto no_memory;
             }
         } else {
             virConfValuePtr vfb, disp;
             char *vfbstr = NULL;
             virBuffer buf = VIR_BUFFER_INITIALIZER;
-            if (def->graphics->type == VIR_DOMAIN_GRAPHICS_TYPE_SDL) {
+            if (def->graphics[0]->type == VIR_DOMAIN_GRAPHICS_TYPE_SDL) {
                 virBufferAddLit(&buf, "type=sdl");
-                if (def->graphics->data.sdl.display)
+                if (def->graphics[0]->data.sdl.display)
                     virBufferVSprintf(&buf, ",display=%s",
-                                      def->graphics->data.sdl.display);
-                if (def->graphics->data.sdl.xauth)
+                                      def->graphics[0]->data.sdl.display);
+                if (def->graphics[0]->data.sdl.xauth)
                     virBufferVSprintf(&buf, ",xauthority=%s",
-                                      def->graphics->data.sdl.xauth);
+                                      def->graphics[0]->data.sdl.xauth);
             } else {
                 virBufferAddLit(&buf, "type=vnc");
                 virBufferVSprintf(&buf, ",vncunused=%d",
-                                  def->graphics->data.vnc.autoport ? 1 : 0);
-                if (!def->graphics->data.vnc.autoport)
+                                  def->graphics[0]->data.vnc.autoport ? 1 : 0);
+                if (!def->graphics[0]->data.vnc.autoport)
                     virBufferVSprintf(&buf, ",vncdisplay=%d",
-                                      def->graphics->data.vnc.port - 5900);
-                if (def->graphics->data.vnc.listenAddr)
+                                      def->graphics[0]->data.vnc.port - 5900);
+                if (def->graphics[0]->data.vnc.listenAddr)
                     virBufferVSprintf(&buf, ",vnclisten=%s",
-                                      def->graphics->data.vnc.listenAddr);
-                if (def->graphics->data.vnc.passwd)
+                                      def->graphics[0]->data.vnc.listenAddr);
+                if (def->graphics[0]->data.vnc.passwd)
                     virBufferVSprintf(&buf, ",vncpasswd=%s",
-                                      def->graphics->data.vnc.passwd);
-                if (def->graphics->data.vnc.keymap)
+                                      def->graphics[0]->data.vnc.passwd);
+                if (def->graphics[0]->data.vnc.keymap)
                     virBufferVSprintf(&buf, ",keymap=%s",
-                                      def->graphics->data.vnc.keymap);
+                                      def->graphics[0]->data.vnc.keymap);
             }
             if (virBufferError(&buf))
                 goto no_memory;
@@ -2260,6 +2465,9 @@ virConfPtr xenXMDomainConfigFormat(virConnectPtr conn,
     }
     VIR_FREE(netVal);
 
+    if (xenXMDomainConfigFormatPCI(conn, conf, def) < 0)
+        goto cleanup;
+
     if (hvm) {
         if (def->nparallels) {
             virBuffer buf = VIR_BUFFER_INITIALIZER;
@@ -2352,12 +2560,10 @@ virDomainPtr xenXMDomainDefineXML(virConnectPtr conn, const char *xml) {
 
     xenUnifiedLock(priv);
 
-#ifndef WITH_XEN_INOTIFY
-    if (xenXMConfigCacheRefresh (conn) < 0) {
+    if (!xenInotifyActive(conn) && xenXMConfigCacheRefresh (conn) < 0) {
         xenUnifiedUnlock(priv);
         return (NULL);
     }
-#endif
 
     if (!(def = virDomainDefParseString(conn, priv->caps, xml,
                                         VIR_DOMAIN_XML_INACTIVE))) {
@@ -2441,9 +2647,7 @@ virDomainPtr xenXMDomainDefineXML(virConnectPtr conn, const char *xml) {
         goto error;
     }
 
-    if ((ret = virGetDomain(conn, def->name, def->uuid)))
-        ret->id = -1;
-
+    ret = virGetDomain(conn, def->name, def->uuid);
     xenUnifiedUnlock(priv);
     return (ret);
 
@@ -2542,10 +2746,8 @@ int xenXMListDefinedDomains(virConnectPtr conn, char **const names, int maxnames
     priv = conn->privateData;
     xenUnifiedLock(priv);
 
-#ifndef WITH_XEN_INOTIFY
-    if (xenXMConfigCacheRefresh (conn) < 0)
+    if (!xenInotifyActive(conn) && xenXMConfigCacheRefresh (conn) < 0)
         goto cleanup;
-#endif
 
     if (maxnames > virHashSize(priv->configCache))
         maxnames = virHashSize(priv->configCache);
@@ -2558,9 +2760,7 @@ int xenXMListDefinedDomains(virConnectPtr conn, char **const names, int maxnames
     virHashForEach(priv->nameConfigMap, xenXMListIterator, &ctx);
     ret = ctx.count;
 
-#ifndef WITH_XEN_INOTIFY
 cleanup:
-#endif
     xenUnifiedUnlock(priv);
     return ret;
 }
@@ -2581,16 +2781,12 @@ int xenXMNumOfDefinedDomains(virConnectPtr conn) {
     priv = conn->privateData;
     xenUnifiedLock(priv);
 
-#ifndef WITH_XEN_INOTIFY
-    if (xenXMConfigCacheRefresh (conn) < 0)
+    if (!xenInotifyActive(conn) && xenXMConfigCacheRefresh (conn) < 0)
         goto cleanup;
-#endif
 
     ret = virHashSize(priv->nameConfigMap);
 
-#ifndef WITH_XEN_INOTIFY
 cleanup:
-#endif
     xenUnifiedUnlock(priv);
     return ret;
 }
@@ -2744,7 +2940,8 @@ xenXMDomainDetachDevice(virDomainPtr domain, const char *xml) {
                 if (i < (def->ndisks - 1))
                     memmove(def->disks + i,
                             def->disks + i + 1,
-                            def->ndisks - (i + 1));
+                            sizeof(*def->disks) *
+                            (def->ndisks - (i + 1)));
                 break;
             }
         }
@@ -2761,7 +2958,8 @@ xenXMDomainDetachDevice(virDomainPtr domain, const char *xml) {
                 if (i < (def->nnets - 1))
                     memmove(def->nets + i,
                             def->nets + i + 1,
-                            def->nnets - (i + 1));
+                            sizeof(*def->nets) *
+                            (def->nnets - (i + 1)));
                 break;
             }
         }
