@@ -18,6 +18,9 @@
 #ifndef WIN32
 #include <sys/wait.h>
 #endif
+#ifdef HAVE_REGEX_H
+#include <regex.h>
+#endif
 #include <unistd.h>
 #include <string.h>
 #include <fcntl.h>
@@ -153,7 +156,6 @@ void virtTestCaptureProgramExecChild(const char *const argv[],
     int i;
     int open_max;
     int stdinfd = -1;
-    int stderrfd = -1;
     const char *const env[] = {
         "LANG=C",
 #if WITH_DRIVER_MODULES
@@ -164,13 +166,10 @@ void virtTestCaptureProgramExecChild(const char *const argv[],
 
     if ((stdinfd = open("/dev/null", O_RDONLY)) < 0)
         goto cleanup;
-    if ((stderrfd = open("/dev/null", O_WRONLY)) < 0)
-        goto cleanup;
 
     open_max = sysconf (_SC_OPEN_MAX);
     for (i = 0; i < open_max; i++) {
         if (i != stdinfd &&
-            i != stderrfd &&
             i != pipefd)
             close(i);
     }
@@ -179,7 +178,7 @@ void virtTestCaptureProgramExecChild(const char *const argv[],
         goto cleanup;
     if (dup2(pipefd, STDOUT_FILENO) != STDOUT_FILENO)
         goto cleanup;
-    if (dup2(stderrfd, STDERR_FILENO) != STDERR_FILENO)
+    if (dup2(pipefd, STDERR_FILENO) != STDERR_FILENO)
         goto cleanup;
 
     /* SUS is crazy here, hence the cast */
@@ -188,8 +187,6 @@ void virtTestCaptureProgramExecChild(const char *const argv[],
  cleanup:
     if (stdinfd != -1)
         close(stdinfd);
-    if (stderrfd != -1)
-        close(stderrfd);
 }
 
 int virtTestCaptureProgramOutput(const char *const argv[],
@@ -409,7 +406,8 @@ int virtTestMain(int argc,
         /* Run once for each alloc, failing a different one
            and validating that the test case failed */
         for (n = 0; n < approxAlloc && (!mp || worker) ; n++) {
-            if ((n % mp) != (worker - 1))
+            if (mp &&
+                (n % mp) != (worker - 1))
                 continue;
             if (!testDebug) {
                 if (mp)
@@ -456,3 +454,56 @@ cleanup:
     virResetLastError();
     return ret;
 }
+
+
+#ifdef HAVE_REGEX_H
+int virtTestClearLineRegex(const char *pattern,
+                           char *str)
+{
+    regex_t reg;
+    char *lineStart = str;
+    char *lineEnd = strchr(str, '\n');
+
+    if (regcomp(&reg, pattern, REG_EXTENDED | REG_NOSUB) != 0)
+        return -1;
+
+    while (lineStart) {
+        int ret;
+        if (lineEnd)
+            *lineEnd = '\0';
+
+
+        ret = regexec(&reg, lineStart, 0, NULL, 0);
+        //fprintf(stderr, "Match %d '%s' '%s'\n", ret, lineStart, pattern);
+        if (ret == 0) {
+            if (lineEnd) {
+                memmove(lineStart, lineEnd + 1, strlen(lineEnd+1) + 1);
+                /* Don't update lineStart - just iterate again on this
+                   location */
+                lineEnd = strchr(lineStart, '\n');
+            } else {
+                *lineStart = '\0';
+                lineStart = NULL;
+            }
+        } else {
+            if (lineEnd) {
+                *lineEnd = '\n';
+                lineStart = lineEnd + 1;
+                lineEnd = strchr(lineStart, '\n');
+            } else {
+                lineStart = NULL;
+            }
+        }
+    }
+
+    regfree(&reg);
+
+    return 0;
+}
+#else
+int virtTestClearLineRegex(const char *pattern ATTRIBUTE_UNUSED,
+                           char *str ATTRIBUTE_UNUSED)
+{
+    return 0;
+}
+#endif
