@@ -124,7 +124,7 @@ static int testCompareFormatXML(const char *xmcfg, const char *xml,
     priv.caps = caps;
     conn->privateData = &priv;
 
-    if (!(conf = virConfReadMem(xmcfgPtr, strlen(xmcfgPtr))))
+    if (!(conf = virConfReadMem(xmcfgPtr, strlen(xmcfgPtr), 0)))
         goto fail;
 
     if (!(def = xenXMDomainConfigParse(conn, conf)))
@@ -209,6 +209,7 @@ mymain(int argc, char **argv)
     DO_TEST("paravirt-new-pvfb", 3);
     DO_TEST("paravirt-new-pvfb-vncdisplay", 3);
     DO_TEST("paravirt-net-e1000", 3);
+    DO_TEST("paravirt-net-vifname", 3);
     DO_TEST("fullvirt-old-cdrom", 1);
     DO_TEST("fullvirt-new-cdrom", 2);
     DO_TEST("fullvirt-utc", 2);
@@ -231,6 +232,7 @@ mymain(int argc, char **argv)
 
     DO_TEST("escape-paths", 2);
     DO_TEST("no-source-cdrom", 2);
+    DO_TEST("pci-devs", 2);
 
     virCapabilitiesFree(caps);
 
@@ -238,4 +240,3 @@ mymain(int argc, char **argv)
 }
 
 VIRT_TEST_MAIN(mymain)
-
